#!/usr/bin/env python

import os
import sys
import abc
import StringIO
from ansible import errors
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser
from InventoryFilters import DummyInventory


filepath = os.path.realpath(__file__)
filepath = "/".join(filepath.split("/")[0:-2])
sys.path.append(filepath)


class FilterModule(object):
    ''' A filter to fix interface's name format '''

    def filters(self):

        inv = DummyInventory.DummyInventory()
        return {
            'dummy_inv': inv.get_inventory
        }



#!/usr/bin/env python

import os
import sys
import ast
import yaml
from collections import namedtuple
from contextlib import contextmanager

from ansible.inventory import Inventory
from ansible.vars import VariableManager
from ansible.parsing.dataloader import DataLoader
from ansible.executor.playbook_executor import PlaybookExecutor


from linchpin.api.utils import yaml2json
from linchpin.api.callbacks import PlaybookCallback
from linchpin.hooks import LinchpinHooks
from linchpin.hooks.state import State
from linchpin.exceptions import LinchpinError


@contextmanager
def suppress_stdout():
    """
    This context manager provides tooling to make Ansible's Display class
    not output anything when used
    """

    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = devnull
        sys.stderr = devnull

        try:
            yield
        finally:
            sys.stdout = old_stdout
            sys.stderr = old_stderr


class LinchpinAPI(object):

    def __init__(self, ctx):
        """
        LinchpinAPI constructor
        :param ctx: context object from api/context.py

        """

        self.ctx = ctx
        base_path = '/'.join(os.path.dirname(__file__).split('/')[0:-2])
        pkg = self.get_cfg(section='lp',
                        key='pkg',
                        default='linchpin')
        self.lp_path = '{0}/{1}'.format(base_path, pkg)
        self.set_evar('from_api', True)

        self.hook_state = None
        self._hook_observers = []
        self.playbook_pre_states = self.get_cfg('playbook_pre_states',
                                                {'up': 'preup', 
                                                 'destroy': 'predestroy'})
        self.playbook_post_states = self.get_cfg('playbook_post_states',
                                                 {'up': 'postup',
                                                  'destroy': 'postdestroy'})
        self.hooks = LinchpinHooks(self)
        self.target_data = {}


    def get_cfg(self, section=None, key=None, default=None):
        """
        Get cfgs value(s) by section and/or key, or the whole cfgs object

        :param section: section from ini-style config file

        :param key: key to get from config file, within section

        :param default: default value to return if nothing is found.

        Does not apply if section is not provided.
        """

        return self.ctx.get_cfg(section=section, key=key, default=default)


    def set_cfg(self, section, key, value):
        """
        Set a value in cfgs. Does not persist into a file,
        only during the current execution.


        :param section: section within ini-style config file

        :param key: key to use

        :param value: value to set into section within config file
        """

        self.ctx.set_cfg(section, key, value)


    def get_evar(self, key=None, default=None):
        """
        Get the current evars (extra_vars)

        :param key: key to use

        :param default: default value to return if nothing is found (default: None)
        """

        return self.ctx.get_evar(key, default)


    def set_evar(self, key, value):
        """
        Set a value into evars (extra_vars). Does not persist into a file,
        only during the current execution.

        :param key: key to use

        :param value: value to set into evars
        """

        self.ctx.set_evar(key, value)


    @property
    def hook_state(self):
        """
        getter function for hook_state property of the API object
        """

        return self.hook_state


    @hook_state.setter
    def hook_state(self, hook_state):
        """
        hook_state property setter , splits the hook_state string in subhook_state and sets
        linchpin.hook_state object

        :param hook_state: valid hook_state string mentioned in linchpin.conf
        """

        # call run_hooks after hook_state is being set
        if hook_state is None:
            return
        else:
#            hook_state = hook_state.split('::')[0]
            self.ctx.log_debug('hook {0} initiated'.format(hook_state))
            self._hook_state = State(hook_state, None, self.ctx)

            for callback in self._hook_observers:
                callback(self._hook_state)


    def bind_to_hook_state(self, callback):
        """
        Function used by LinchpinHooksclass to add callbacks

        :param callback: callback function
        """

        self._hook_observers.append(callback)


    def set_magic_vars(self):
        """
        Function inbuilt to set magic vars for ansible context
        """

        try:
            t_f = open(self.get_evar("topology"), "r").read()
            t_f = yaml.load(t_f)
            topology_name = t_f["topology_name"]
        except Exception as e:
            self.ctx.log_info("{0}".format(str(e)))
            topology_name = self.get_evar("topology").split("/")[-1]
            # defaults to file name if there is any error
            topology_name = topology_name.split(".")[-2]
        inv_file = '{0}/{1}/{2}{3}'.format(self.ctx.workspace,
                        self.get_evar('inventories_folder'),
                        topology_name,
                        self.get_cfg('extensions','inventory' ,'inventory')
                    )
        self.set_evar('inventory_file', inv_file)
        self.set_evar('topology_name', topology_name)

    def run_playbook(self, pinfile, targets='all', playbook='up'):
        """
        This function takes a list of targets, and executes the given
        playbook (provison, destroy, etc.) for each provided target.

        :param pinfile: Provided PinFile, with available targets,

        :param targets: A tuple of targets to run. (default: 'all')
        """

        pf = yaml2json(pinfile)

        # playbooks check whether from_api is defined
        self.ctx.log_debug('from_api: {0}'.format(self.get_evar('from_api')))

        # playbooks check whether from_cli is defined
        # if not, vars get loaded from linchpin.conf
        self.set_evar('from_api', True)
        self.set_evar('lp_path', self.lp_path)

        #do we display the ansible output to the console?
        ansible_console = False
        if self.ctx.cfgs.get('ansible'):
            ansible_console = ast.literal_eval(self.ctx.cfgs['ansible'].get('console', 'False'))

        if not ansible_console:
            ansible_console = self.ctx.verbose

        self.set_evar('default_resources_path', '{0}/{1}'.format(
                                            self.ctx.workspace,
                                            self.get_evar('resources_folder',
                                                    default='resources')))

        # playbooks still use this var, keep it here
        self.set_evar('default_inventories_path', '{0}/{1}'.format(
                                        self.ctx.workspace,
                                        self.get_evar('inventories_folder',
                                                default='inventories')))

        # add this because of magic_var evaluation in ansible
        self.set_evar('inventory_dir', self.get_evar(
                                        'default_inventories_path',
                                        default='inventories'))

        self.set_evar('state', 'present')


        if playbook == 'destroy':
            self.set_evar('state', 'absent')

        results = {}

        # determine what targets is equal to
        if (set(targets) == set(pf.keys()).intersection(targets) and
                                                        len(targets) > 0):
            pass
        elif len(targets) == 0:
            targets = set(pf.keys()).difference()
        else:
            raise LinchpinError("One or more Invalid targets found")


        for target in targets:
            self.set_evar('topology', self.find_topology(
                    pf[target]["topology"]))

            if 'layout' in pf[target]:
                self.set_evar('layout_file', (
                    '{0}/{1}/{2}'.format(self.ctx.workspace,
                                self.get_evar('layouts_folder'),
                                pf[target]["layout"])))

            # parse topology_file and set inventory_file
            self.set_magic_vars()

            # set the current target data
            self.target_data = pf[target]
            self.target_data["extra_vars"] = self.get_evar()

            # note : changing the state triggers the hooks
            self.pb_hooks = self.get_cfg('hookstates', playbook)
            self.ctx.log_debug('calling: {0}{1}'.format('pre', playbook))

            if 'pre' in self.pb_hooks:
                self.hook_state = '{0}{1}'.format('pre', playbook)

            #invoke the appropriate playbook
            return_code, results[target] = self._invoke_playbook(
                                            playbook=playbook,
                                            console=ansible_console)

            if not return_code:
                self.ctx.log_state("Action '{0}' on Target '{1}' is "
                        "complete".format(playbook, target))

            # FIXME Check the result[target] value here, and fail if applicable.
            # It's possible that a flag might allow more targets to run, then
            # return an error code at the end.

            if 'post' in self.pb_hooks:
                self.hook_state = '{0}{1}'.format('post', playbook)

        return results


    def lp_rise(self, pinfile, targets='all'):
        """
        DEPRECATED

        An alias for lp_up. Used only for backward compatibility.
        """

        self.lp_up(pinfile, targets)


    def lp_up(self, pinfile, targets='all'):
        """
        This function takes a list of targets, and provisions them according
        to their topology. If an layout argument is provided, an inventory
        will be generated for the provisioned nodes.

        :param pinfile:
            Provided PinFile, with available targets,

        :param targets:
            A tuple of targets to provision.
        """

        return self.run_playbook(pinfile, targets, playbook="up")


    def lp_drop(self, pinfile, targets):
        """
        DEPRECATED

        An alias for lp_destroy. Used only for backward compatibility.
        """

        return self.lp_destroy(pinfile, targets)


    def lp_destroy(self, pinfile, targets='all'):
        """
        This function takes a list of targets, and performs a destructive
        teardown, including undefining nodes, according to the target.

        .. seealso:: lp_down - currently unimplemented

        :param pinfile:
            Provided PinFile, with available targets,

        :param targets:
            A tuple of targets to destroy.
        """

        return self.run_playbook(pinfile, targets, playbook="destroy")


    def lp_down(self, pinfile, targets='all'):
        """
        This function takes a list of targets, and performs a shutdown on
        nodes in the target's topology. Only providers which support shutdown
        from their API (Ansible) will support this option.

        CURRENTLY UNIMPLEMENTED

        .. seealso:: lp_destroy

        :param pinfile:
            Provided PinFile, with available targets,

        :param targets:
            A tuple of targets to provision.
        """

        pass


    def find_topology(self, topology):
        """
        Find the topology to be acted upon. This could be pulled from a
        registry.

        :param topology:
            name of topology from PinFile to be loaded

        """

        topo_path = os.path.realpath('{0}/{1}'.format(
                self.ctx.workspace,
                self.get_evar('topologies_folder', 'topologies')))

        topos = os.listdir(topo_path)

        if topology in topos:
            return os.path.realpath('{0}/{1}'.format(topo_path, topology))

        raise LinchpinError('Topology {0} not found in workspace'.format(topology))



    def _invoke_playbook(self, playbook='up', console=True):
        """
        Uses the Ansible API code to invoke the specified linchpin playbook

        :param playbook: Which ansible playbook to run (default: 'up')
        :param console: Whether to display the ansible console (default: True)
        """

        pb_path = '{0}/{1}'.format(self.lp_path,
                            self.ctx.get_evar('playbooks_folder', 'provision'))
        module_path = '{0}/{1}/'.format(pb_path, self.get_cfg('lp', 'module_folder', 'library'))
        playbook_path = '{0}/{1}'.format(pb_path, self.get_cfg('playbooks', playbook, 'site.yml'))

        loader = DataLoader()
        variable_manager = VariableManager()
        variable_manager.extra_vars = self.get_evar()
        inventory = Inventory(loader=loader,
                              variable_manager=variable_manager,
                              host_list=[])
        passwords = {}
        #utils.VERBOSITY = 4

        Options = namedtuple('Options', ['listtags',
                                         'listtasks',
                                         'listhosts',
                                         'syntax',
                                         'connection',
                                         'module_path',
                                         'forks',
                                         'remote_user',
                                         'private_key_file',
                                         'ssh_common_args',
                                         'ssh_extra_args',
                                         'sftp_extra_args',
                                         'scp_extra_args',
                                         'become',
                                         'become_method',
                                         'become_user',
                                         'verbosity',
                                         'check'])

        options = Options(listtags=False,
                          listtasks=False,
                          listhosts=False,
                          syntax=False,
                          connection='ssh',
                          module_path=module_path,
                          forks=100,
                          remote_user='test',
                          private_key_file=None,
                          ssh_common_args=None,
                          ssh_extra_args=None,
                          sftp_extra_args=None,
                          scp_extra_args=None,
                          become=False,
                          become_method='sudo',
                          become_user='root',
                          verbosity=0,
                          check=False)

        pbex = PlaybookExecutor(playbooks=[playbook_path],
                                inventory=inventory,
                                variable_manager=variable_manager,
                                loader=loader,
                                options=options,
                                passwords=passwords)

        if not console:
            results = {}
            return_code = 0

            cb = PlaybookCallback()

            with suppress_stdout():
                pbex._tqm._stdout_callback = cb

            return_code = pbex.run()
            results = cb.results

            return return_code, results
        else:
            # the console only returns a return_code
            return_code = pbex.run()
            return return_code, None


#    def lp_topo_list(self, upstream=None):
#        """
#        search_order : list topologies from upstream if mentioned
#                       list topologies from current folder
#        """
#        if upstream is None:
#            t_files = list_files(self.base_path + "/examples/topology/")
#            return t_files
#        else:
#            print("getting from upstream")
#            g = GitHub(upstream)
#            t_files = []
#            repo_path = LinchpinAPI.UPSTREAM_EXAMPLES_PATH + "/topology"
#            files = g.list_files(repo_path)
#            return files
#
#    REPLACED ABOVE (but it doesn't do the registry lookup yet)
#    def find_topology(self, topology, topolgy_registry):
#        print("searching for topology in configured workspace: "+self.context.workspace)
#        try:
#            topos = os.listdir(self.context.workspace+"/topologies")
#            if topology in topos:
#                return os.path.abspath(self.context.workspace+"/topologies/"+topology)
#        except OSError as e:
#            click.echo(str(e))
#            click.echo("topologies directory  not found in workspace.")
#        except Exception as e:
#            click.echo(str(e))
#        click.echo("Searching for topology in linchpin package.")
#        topos = self.lp_topo_list()
#        topos = [t["name"] for t in topos]
#        if topology in topos:
#            click.echo("Topology file found in linchpin package.")
#            click.echo("Copying it to workspace")
#            self.lp_topo_get(topology)
#            return os.path.abspath(self.context.workspace+"/topologies/"+topology)
#        click.echo("Topology file not found")
#        click.echo("Searching for topology from upstream")
#        # currently supports only one topology registry per PinFile
#        if topology_registry:
#            try:
#                topos = self.lp_topo_list(topology_registry)
#                topos = [x["name"] for x in topos]
#                if topology in topos:
#                    click.echo("Found topology in registry")
#                    click.echo("Fetching topology from registry")
#                    self.lp_topo_get(topology, topology_registry)
#                    return os.path.abspath(self.context.workspace+"/topologies/"+topology)
#            except Exception as e:
#                click.echo("Exception occurred "+str(e))
#        raise IOError("Topology file not found. Invalid topology reference in PinFile")
#
#    def find_layout(self, layout, layout_registry=None):
#        print("searching for layout in configured workspace: "+self.context.workspace)
#        try:
#            layouts = os.listdir(self.context.workspace+"/layouts")
#            if layout in layouts:
#                return os.path.abspath(self.context.workspace+"/layouts/"+layout)
#        except OSError as e:
#            click.echo(str(e))
#            click.echo("layouts directory  not found in workspace.")
#        except Exception as e:
#            click.echo(str(e))
#        click.echo("Searching for layout in linchpin package.")
#        layouts = self.lp_layout_list()
#        layouts = [t["name"] for t in layouts]
#        if layout in layouts:
#            click.echo("layout file found in linchpin package.")
#            click.echo("Copying it to workspace")
#            self.lp_layout_get(layout)
#            return os.path.abspath(self.context.workspace+"/layouts/"+layout)
#        click.echo("layout file not found")
#        click.echo("Searching for layout from upstream")
#        # currently supports only one layout registry per PinFile
#        if layout_registry:
#            try:
#                layouts = self.lp_layout_list(layout_registry)
#                layouts = [x["name"] for x in layouts]
#                if layout in layouts:
#                    click.echo("Found layout in registry")
#                    click.echo("Fetching layout from registry")
#                    self.lp_layout_get(layout, layout_registry)
#                    return os.path.abspath(self.context.workspace+"/layouts/"+layout)
#            except Exception as e:
#                click.echo("Exception occurred "+str(e))
#        raise IOError("layout file not found. Invalid layout reference in PinFile")
#
#    def lp_topo_get(self, topo, upstream=None):
#        """
#        search_order : get topologies from upstream if mentioned
#                       get topologies from core package
#        # need to add checks for ./topologies
#        """
#        if upstream is None:
#            pkg_file_path = self.base_path + "/examples/topology/" + topo
#            return open(pkg_file_path).read()
#            #get_file(self.base_path + "/examples/topology/" + topo,
#            #         "./topologies/")
#        else:
#            g = GitHub(upstream)
#            repo_path = LinchpinAPI.UPSTREAM_EXAMPLES_PATH + "/topology"
#            files = g.list_files(repo_path)
#            link = filter(lambda link: link['name'] == topo, files)
#            link = link[0]["download_url"]
#            return requests.get(link).text
#            #get_file(link, "./topologies", True)
#            #return link
#
#    def lp_layout_list(self, upstream=None):
#        """
#        search_order : list layouts from upstream if mentioned
#                       list layouts from core package
#        """
#        if upstream is None:
#            l_files = list_files(self.base_path + "examples/layouts/")
#            return l_files
#        else:
#            g = GitHub(upstream)
#            l_files = []
#            repo_path = LinchpinAPI.UPSTREAM_EXAMPLES_PATH + "/layouts"
#            files = g.list_files(repo_path)
#            return files
#
#    def lp_layout_get(self, layout, upstream=None):
#        """
#        search_order : get layouts from upstream if mentioned
#                       get layouts from core package
#        """
#        if upstream is None:
#            pkg_file_path = self.base_path + "/examples/layouts/" + layout
#            return open(pkg_file_path, "r").read()
#            #get_file(self.base_path + "/examples/layouts/" + layout,
#            #         "./layouts/")
#        else:
#            g = GitHub(upstream)
#            repo_path = LinchpinAPI.UPSTREAM_EXAMPLES_PATH + "/layouts"
#            files = g.list_files(repo_path)
#            link = filter(lambda link: link['name'] == layout, files)
#            link = link[0]["download_url"]
#            return requests.get(link).text
#
#
#
#    def lp_validate_topology(self, topology):
#        self.evars = {}
#        self.evars["schema"] = self.base_path + "/schemas/schema_v3.json"
#        self.evars["data"] = topology
#        result = invoke_linchpin(self.base_path, self.evars,
#                                 "SCHEMA_CHECK", console=True)
#        print(result)
#        return result
#
#
#    def lp_invgen(self, topoout, layout, invout, invtype):
#        """ invgen module of linchpin cli """
#        self.evars = {}
#        self.evars['linchpin_config'] = self.get_config_path()
#        self.evars['output'] = os.path.abspath(topoout)
#        self.evars['layout'] = os.path.abspath(layout)
#        self.evars['inventory_type'] = invtype
#        self.evars['inventory_output'] = invout
#        result = invoke_linchpin(self.base_path,
#                                 self.evars,
#                                 "INVGEN",
#                                 console=True)
#
#    def lp_test(self, topo, layout, pf):
#        """ test module of linchpin.api"""
#        self.evars = {}
#        self.evars['data'] = topo
#        self.evars['schema'] = self.base_path + "/schemas/schema_v3.json"
#        result = invoke_linchpin(self.base_path, self.evars, "TEST", console=True)
#        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Samvaran Kashyap Rallabandi -  <srallaba@redhat.com>
#
# Topology validator for Ansible based infra provsioning tool linch-pin

from ansible.module_utils.basic import *
import datetime
import sys
import json
import os
import shlex
import tempfile
import yaml
import jsonschema
from jsonschema import validate

DOCUMENTATION = '''
---
version_added: "0.1"
module: topo_check
short_description: Topology validator module in ansible
description:
  - This module allows a user to validate the yaml/json
    provisioning topologies against json schema.

options:
  data_file:
    description:
      path to topology file can be in json/yaml format
    required: true
  data_format:
    description:
      format of the topology file
    default: yaml
  schema_file:
    description:
      Schema to be validated against
    required: true

author: Samvaran Kashyap Rallabandi -
'''


class JSONSchema:
    def __init__(self, data_file_path, schema_file_path):
        self.data_file = data_file_path
        self.schema_file = schema_file_path

    def validate(self):
        data = self.get_data(self.data_file)
        schema = open(self.schema_file).read()

        try:
            result = jsonschema.validate(json.loads(data), json.loads(schema))
            return (True, json.loads(data))
        except jsonschema.ValidationError as e:
            return (False, "ValidationError: {0}".format(e.message))
        except jsonschema.SchemaError as e:
            return (False, "SchemaError: {0}".format(e))
        except Exception as e:
            return (False, "Unknown Error: {0}".format(e))

    def get_data(self, file_path):
        ext = file_path.split(".")[-1]
        if (ext == "yml" or ext == "yaml"):
            fd = open(file_path)
            return json.dumps(yaml.safe_load(fd))
        if (ext == "json"):
            return open(self.topo_file).read()
        else:
            return {"error": "Invalid File Format"}


def check_file_paths(module, *args):
    for file_path in args:
        if not os.path.exists(file_path):
            msg = "File not found %s not found" % (file_path)
            module.fail_json(msg=msg)
        if not os.access(file_path, os.R_OK):
            msg = "File not accesible %s not found" % (file_path)
            module.fail_json(msg=msg)
        if os.path.isdir(file_path):
            msg = "Recursive directory not supported  %s " % (file_path)
            module.fail_json(msg=msg)


def validate_grp_names(data):
    res_grps = data['resource_groups']
    if 'resource_group_vars' in data.keys():
        res_grp_vars = data['resource_group_vars']
    else:
        res_grp_vars = []
    res_grp_names = [x['resource_group_name'] for x in res_grps]
    if len(res_grp_vars) > 0:
        res_grp_vars = [x['resource_group_name'] for x in res_grp_vars]
    dup_grp_names = set(res_grp_names)
    dup_grp_vars = set(res_grp_vars)
    if len(dup_grp_vars) != len(res_grp_vars) or \
       len(dup_grp_names) != len(res_grp_names):
        msg = "error: duplicate names found in resource_group_name \
               attributes please check the results for duplicate names"
        return {"msg": msg, "result": str(dup_grp_names)+str(dup_grp_vars)}
    else:
        return True


def validate_values(module, data_file_path):
    data = open(data_file_path).read()
    data = yaml.safe_load(data)
    status = validate_grp_names(data)
    if not status:
        module.fail_json(msg="%s" % (json.dumps(status)))
    else:
        return status


def main():
    module = AnsibleModule(
    argument_spec={
            'data': {'required': True, 'aliases': ['topology']},
            'schema': {'required': True},
            'data_format': {'required': False,'choices':['json','yaml','yml']},
        },
        required_one_of=[],
        supports_check_mode=True
        )
    data_file_path = os.path.expanduser(module.params['data'])
    schema_file_path = os.path.expanduser(module.params['schema'])
    check_file_paths(module, data_file_path, schema_file_path)
    validate_values(module, data_file_path)
    schema_obj = JSONSchema(data_file_path, schema_file_path)

    status, out = schema_obj.validate()

#    module.fail_json(msg=out)

    if status:
        changed = True
        module.exit_json(isvalid=changed, out=out)
    else:
        resp = {"path": data_file_path,
                "schema": schema_file_path,
                "output": out}

        module.fail_json(msg=resp)

main()



import os
import sys
import subprocess
from action_manager import ActionManager
from cerberus import Validator

from linchpin.exceptions import HookError


class SubprocessActionManager(ActionManager):

    def __init__(self, name, action_data, target_data, **kwargs):

        """
        SubprocessActionManager constructor
        :param name: Name of Action Manager , ( ie., shell)
        :param action_data: dictionary of action_block consists of set of actions
        example:
        - name: hookname
          type: shell
          actions:
            - echo " this is post down operation Hello hai how r u ?"
            - test.sh
        :param target_data: Target specific data defined in PinFile
        :param kwargs: anyother keyword args passed as metadata
        """

        self.name = name
        self.action_data = action_data
        self.target_data = target_data
        self.context = kwargs.get('context', True)
        self.kwargs = kwargs

    def validate(self):

        """
        Validates the action_block based on the cerberus schema
        example:
        action_block :: sample ::
        - name: manipulate_inventory
          type: shell
          path: /tmp/shellscripts
          actions:
            - thisisshell.sh
        """
        schema= { 'name': { 
                           'type':'string',
                           'required': True
                          },
                  'type': { 
                           'type': 'string',
                           'allowed': ['shell', 'subprocess']
                          },
                  'path': {
                           'type': 'string',
                           'required': False
                          },
                  'context': {
                           'type': 'boolean',
                           'required': False
                           },
                  'actions': { 
                               'type': 'list',
                               'schema': {
                                          'type': 'string'
                                         },
                               'required': True
                             }
        }
        v = Validator(schema)
        status = v.validate(self.action_data)
        if not status:
            raise HookError("Invalid syntax: LinchpinHook:"+str((v.errors)))
        else:
            return status

    def load(self):
        
        """
        Adds the shell script to the os path if mentioned
        """

        # set os.environpath if exists
        if self.action_data.has_key("path"):
            os.environ["PATH"] += ":"+self.action_data["path"]

    def add_context_params(self, action):
        
        """
        Adds ctx params to the action_block run when context is true
        :param file_path: path to the script
        :param context: whether the context params are to be included or not
        """

        command = action
        for key in self.target_data:
            command += " %s=%s " %(key, self.target_data[key])
        return command

    def execute(self):
        
        """
        Executes the action_block in the PinFile
        """

        self.load()
        for action in self.action_data["actions"]:
            if self.context:
                command = self.add_context_params(action)
            else:
                command = action
            proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
            proc.wait()
            for line in proc.stdout:
                print(line)



import os
import sys
import logging
import json
import io

import xml.dom.minidom
import xml.etree.ElementTree as eT

from bkr.client.task_watcher import watch_tasks
from bkr.client import BeakerCommand, BeakerWorkflow, BeakerJob, \
    BeakerRecipeSet, BeakerRecipe, conf
from bkr.common.hub import HubProxy
from bkr.common.pyconfig import PyConfigParser

# WANT_JSON
# We want JSON input from Ansible. Please give it to us

LOG = logging.getLogger(__name__)
LOG.setLevel(logging.DEBUG)
BEAKER_CONF = \
    (os.environ.get('BEAKER_CONF', '/etc/beaker/client.conf'))


class BkrConn(object):
    """
        Make connection to Beaker
    """
    enabled = True
    doc = xml.dom.minidom.Document()

    def __init__(self, logger=None, conf=None, **kwargs):
        self.conf = PyConfigParser()
        default_config = os.path.expanduser(BEAKER_CONF)
        self.conf.load_from_file(default_config)
        self.hub = HubProxy(logger=logger, conf=self.conf, **kwargs)


class BkrFactory(BkrConn):
    """
        Simple way to provision a job to the Beaker scheduler
    """
    def __init__(self, *args, **kwargs):
        super(BkrFactory, self).__init__(*args, **kwargs)

    def provision(self, *args, **kwargs):
        """
            provision resources in Beaker
        """
        # Break down kwargs for debug, dryrun, wait, recipesets, and whiteboard
        debug = kwargs.get("debug", False)
        dryrun = kwargs.get("dryrun", False)
        wait = kwargs.get("wait", False)
        recipesets = kwargs.get("recipesets", [])

        # Create Job
        job = BeakerJob(*args, **kwargs)

        # Add All Host Requirements
        for recipeset in recipesets:
            kwargs = self.create_recipesets(recipeset, **kwargs)
            family = kwargs.get("family", None)
            distro = kwargs.get("distro", None)
            task_params = kwargs.get("taskparam", [])
            arch = kwargs.get("arch", None)
            ks_meta = kwargs.get("ks_meta", "")
            method = kwargs.get("method", "nfs")
            priority = kwargs.get("priority", "Normal")
            hostrequires = kwargs.get("hostrequires", [])

            # Tasks and harnesses
            if 'harness' in ks_meta:
                # Disable report plugins
                task_params.append("RSTRNT_DISABLED=01_dmesg_check "
                                   "10_avc_check")

                # Reserve the system after its installed
                kwargs.update({"reserve": True})
                # We don't need to run a task but beaker needs one.
                requested_tasks = \
                    [{'arches': [], 'name': '/distribution/dummy'}]

                # if no repos are defined but with no value use default repo
                # This default will not work for Fedora
                repos = kwargs.get(
                    "repos",
                    ["http://file.bos.redhat.com/~bpeck/restraint/rhel"
                     "$releasever"]
                )
                kwargs.update({'repo': repos})
            else:
                requested_tasks = \
                    [{'arches': [], 'name': '/distribution/dummy'},
                     {'arches': [], 'name': '/distribution/reservesys'}]

            # Update defaults
            kwargs.update({"suppress_install_task": True})
            kwargs.update({"method": method})
            kwargs.update({"priority": priority})

            # Must have family or distro
            if not family and not distro and not arch:
                sys.stderr.write("No Family or Distro and arch specified\n")
                sys.exit(1)

            if not requested_tasks:
                sys.stderr.write("You must specify a task to run\n")
                sys.exit(1)

            # Create Workflow
            wrkflow = BeakerWorkflow('BeakerWorkflow')

            # Create Base Recipe
            recipe_template = BeakerRecipe(*args, **kwargs)

            # Add Host Requirements
            if 'force' in hostrequires:
                # hostRequires element is created by BeakerRecipe, use it
                hostrequires_node = recipe_template.node.getElementsByTagName('hostRequires')[0]
                # all other filters are ignored if the hostname is forced, so the use of 'force'
                # is mutually exclusive with the use of any other 'hostRequires' filters
                hostrequires_node.setAttribute('force', hostrequires['force'])
            else:
                for requirement in hostrequires:
                    # If force is not used, a requirement can be any number of differently
                    # formatted XML elements, each with their own combination of element name and
                    # valid attrs. So, the best we can do is generate XML based on the input, and
                    # only the "tag" key is required.
                    tag_name = requirement.pop('tag')
                    requirement_node = self.doc.createElement(tag_name)
                    for attr, value in requirement.items():
                        # Force all values to str, which the XML writer expects.
                        requirement_node.setAttribute(attr, str(value))
                    # use the BeakerRecipe API to add the element
                    recipe_template.addHostRequires(requirement_node)

            # Add Distro Requirements
            recipe_template.addBaseRequires(*args, **kwargs)
            arch_node = self.doc.createElement('distro_arch')
            arch_node.setAttribute('op', '=')
            arch_node.setAttribute('value', arch)
            recipe_set = BeakerRecipeSet(**kwargs)
            if wrkflow.multi_host:
                for i in range(self.n_servers):
                    recipe_set.addRecipe(wrkflow.processTemplate(
                        recipe_template,
                        requested_tasks,
                        taskParams=task_params,
                        distroRequires=arch_node,
                        role='SERVERS',
                        **kwargs))
                for i in range(self.n_clients):
                    recipe_set.addRecipe(wrkflow.processTemplate(
                        recipe_template,
                        requested_tasks,
                        taskParams=task_params,
                        distroRequires=arch_node,
                        role='CLIENTS',
                        **kwargs))
            else:
                recipe_set.addRecipe(wrkflow.processTemplate(
                    recipe_template,
                    requested_tasks,
                    taskParams=task_params,
                    distroRequires=arch_node,
                    **kwargs))
            job.addRecipeSet(recipe_set)

        # jobxml
        jobxml = job.toxml(**kwargs)

        if debug:
            LOG.debug(jobxml)

        self.submitted_jobs = []
        is_failed = False

        if not dryrun:
            try:
                self.submitted_jobs.append(self.hub.jobs.upload(jobxml))
                LOG.info("Submitted: %s" % self.submitted_jobs)
                return self.submitted_jobs
            except (KeyboardInterrupt, SystemExit):
                raise
            except Exception, ex:
                is_failed = True
                sys.stderr.write('Exception: %s\n' % ex)
            if wait:
                is_failed |= watch_tasks(self.hub, self.submitted_jobs)
        return is_failed

    def create_recipesets(self, recipeset, **kwargs):
        kwargs = {}
        for key, values in recipeset.iteritems():
            kwargs.update({key: values})
        return kwargs

    def check_jobs(self, jobs):
        """
            Get state of a job in Beaker
        """
        results = {}
        resources = []
        bkrcmd = BeakerCommand('BeakerCommand')
        bkrcmd.check_taskspec_args(jobs)
        for task in jobs:
            myxml = self.hub.taskactions.to_xml(task)
            myxml = myxml.encode('utf8')
            LOG.debug(xml.dom.minidom.parseString(myxml)
                         .toprettyxml(encoding='utf8'))
            root = eT.fromstring(myxml)
            # TODO: Using getiterator() since its backward compatible
            # with Python 2.6
            # This is deprectated in 2.7 and we should be using iter()
            for job in root.getiterator('job'):
                results.update({'job_id': job.get('id'),
                                'results': job.get('result')})
            for recipe in root.getiterator('recipe'):
                resources.append({'family': recipe.get('family'),
                                  'distro': recipe.get('distro'),
                                  'arch': recipe.get('arch'),
                                  'variant': recipe.get('variant'),
                                  'system': recipe.get('system'),
                                  'status': recipe.get('status'),
                                  'result': recipe.get('result')})
                results.update({'resources': resources})
        return results

    def cancel_jobs(self, jobs, msg):
        """
            Cancel job in Beaker
        """
        for task in jobs:
            self.hub.taskactions.stop(task, 'cancel', msg)
            LOG.info("Cancel job %s in Beaker" % task)


def main():
    module = AnsibleModule(argument_spec={
        'whiteboard': {'required': False, 'type': 'str'},
        'recipesets': {'required': True, 'type': 'list'},
        'job_group': {'required': True, 'type': 'str'},
        'state': {'default': 'present', 'choices': ['present', 'absent']},
        'cancel_message': {'default': 'Job canceled by Ansible'}
    })
    params = type('Args', (object,), module.params)
    factory = BkrFactory()
    logs = []
    job_ids = []
    for recipeset in params.recipesets:
        for x in range(0, recipeset['count']):
            if params.state == 'present':
                # Make provision
                job_id = factory.provision(debug=True, wait=True,\
                                           recipesets=[recipeset],\
                                           job_group=params.job_group,
                                           whiteboard=params.whiteboard)
                job_ids.extend(job_id)
            else:
                factory.cancel_jobs(recipeset['ids'], params.cancel_message)
    if params.state == 'present':
        parsed_ids = []
        for id_string in job_ids:
            parsed_ids.append(id_string[2:])
        module.exit_json(success=False, ids=parsed_ids, changed=True)
    else:
        module.exit_json(success=True, changed=True)


# import module snippets
from ansible.module_utils.basic import *
main()



#!/usr/bin/python
# Copyright 2017 RedHat Inc. All Rights Reserved.
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>

DOCUMENTATION = '''
---
module: openshift
version_added: "2.4"
short_description: Manage OpenShift resources.
description:
    - This module can manage OpenShift resources on an existing cluster using
      the OpenShift server API. Users can specify in-line API data, or
      specify an existing OpenShift YAML file. Currently, this module,
        Only supports 'strategic merge' for update, http://goo.gl/fCPYxT
        SSL certs are not working, use 'validate_certs=off' to disable
options:
  api_endpoint:
    description:
      - The IPv4 API endpoint of the OpenShift cluster.
    required: true
    default: null
    aliases: ["endpoint"]
  inline_data:
    description:
      - The OpenShift YAML data to send to the API I(endpoint). This option is
        mutually exclusive with C('file_reference').
    required: true
    default: null
  file_reference:
    description:
      - Specify full path to a OpenShift YAML file to send to API I(endpoint).
        This option is mutually exclusive with C('inline_data').
    required: false
    default: null
  certificate_authority_data:
    description:
      - Certificate Authority data for OpenShift server. Should be in either
        standard PEM format or base64 encoded PEM data. Note that certificate
        verification is broken until ansible supports a version of
        'match_hostname' that can match the IP address against the CA data.
    required: false
    default: null
  state:
    description:
      - The desired action to take on the OpenShift data.
    required: true
    default: "present"
    choices: ["present", "absent", "update", "replace"]
  api_token:
    description:
      - The OpenShift token for use when authenticating against the API. This
        can be acquired with the OpenShift CLI client ("oc") after loggin in
        by executing the command "oc whoami --token"
    required: true
    aliases: ["token"]
  validate_certs:
    description:
      - Enable/disable certificate validation. Note that this is set to
        C(false) until Ansible can support IP address based certificate
        hostname matching (exists in >= python3.5.0).
    required: false
    default: false

author: "Greg Hellings @greg-hellings <greg.hellings@gmail.com>"
'''

EXAMPLES = '''
# Create a new namespace with in-line YAML.
- name: Create a openshift namespace
  openshift:
    api_endpoint: 123.45.67.89
    api_token: sometokenstringhere
    inline_data:
      kind: Namespace
      apiVersion: v1
      metadata:
        name: ansible-test
        labels:
          label_env: production
          label_ver: latest
        annotations:
          a1: value1
          a2: value2
    state: present

# Create a new namespace from a YAML file.
- name: Create a openshift namespace
  openshift:
    api_endpoint: 123.45.67.89
    api_token: sometokenstringhere
    file_reference: /path/to/create_namespace.yaml
    state: present
'''

RETURN = '''
# Example response from creating a OpenShift Namespace.
api_response:
    description: Raw response from OpenShift API, content varies with API.
    returned: success
    type: dictionary
    contains:
        apiVersion: "v1"
        kind: "Namespace"
        metadata:
            creationTimestamp: "2016-01-04T21:16:32Z"
            name: "test-namespace"
            resourceVersion: "509635"
            selfLink: "/api/v1/namespaces/test-namespace"
            uid: "6dbd394e-b328-11e5-9a02-42010af0013a"
        spec:
            finalizers:
                - openshift
        status:
            phase: "Active"
'''

import yaml
import base64

from copy import copy

############################################################################
############################################################################
# For API coverage, this Anislbe module provides capability to operate on
# all OpenShift objects that support a "create" call (except for 'Events').
# In order to obtain a valid list of OpenShift objects, the v1 spec file
# was referenced and the below python script was used to parse the JSON
# spec file, extract only the objects with a description starting with
# 'create a'. The script then iterates over all of these base objects
# to get the endpoint URL and was used to generate the KIND_URL map.
#
#import json
#from urllib2 import urlopen
#
#apis = {}
#cluster_apis = {}
#
#def load_api(api):
#    v1 = json.load(api)
#
#    for a in v1['apis']:
#        p = a['path']
#        for o in a['operations']:
#            if o["summary"].startswith("create a") and o["type"] != "v1.Event":
#                if "{namespace}" in p:
#                    apis[o["type"]] = p
#                else:
#                    cluster_apis[o["type"]] = p
#
#def print_kind_url_map():
#    results = ['"{0}": "{1}"'.format(a[3:].lower(), apis[a])
#               for a in apis.keys()]
#    cluster_results = ['"{0}": "{1}"'.format(a[3:].lower(), cluster_apis[a])
#                       for a in cluster_apis.keys()]
#    results.sort()
#    cluster_results.sort()
#    print "KIND_URL = {"
#    print ",\n".join(results)
#    print "}"
#    print "CLUSTER_URL = {"
#    print ",\n".join(cluster_results)
#    print "}"
#
#if __name__ == '__main__':
#    k = urlopen("https://raw.githubusercontent.com/openshift"
#               "/origin/master/api/swagger-spec/api-v1.json")
#    o = urlopen("https://raw.githubusercontent.com/openshift"
#               "/origin/master/api/swagger-spec/oapi-v1.json")
#    load_api(k)
#    load_api(o)
#    print_kind_url_map()
############################################################################
############################################################################

KIND_URL = {
    "binding": "/api/v1/namespaces/{namespace}/bindings",
    "build": "/oapi/v1/namespaces/{namespace}/builds",
    "buildconfig": "/oapi/v1/namespaces/{namespace}/buildconfigs",
    "configmap": "/api/v1/namespaces/{namespace}/configmaps",
    "deploymentconfig": "/oapi/v1/namespaces/{namespace}/deploymentconfigs",
    "deploymentconfigrollback": "/oapi/v1/namespaces/{namespace}/deploymentconfigrollbacks",
    "egressnetworkpolicy": "/oapi/v1/namespaces/{namespace}/egressnetworkpolicies",
    "imagestream": "/oapi/v1/namespaces/{namespace}/imagestreams",
    "imagestreamimport": "/oapi/v1/namespaces/{namespace}/imagestreamimports",
    "imagestreammapping": "/oapi/v1/namespaces/{namespace}/imagestreammappings",
    "imagestreamtag": "/oapi/v1/namespaces/{namespace}/imagestreamtags",
    "limitrange": "/api/v1/namespaces/{namespace}/limitranges",
    "localresourceaccessreview": "/oapi/v1/namespaces/{namespace}/localresourceaccessreviews",
    "localsubjectaccessreview": "/oapi/v1/namespaces/{namespace}/localsubjectaccessreviews",
    "persistentvolumeclaim": "/api/v1/namespaces/{namespace}/persistentvolumeclaims",
    "pod": "/api/v1/namespaces/{namespace}/pods",
    "podsecuritypolicyreview": "/oapi/v1/namespaces/{namespace}/podsecuritypolicyreviews",
    "podsecuritypolicyselfsubjectreview": "/oapi/v1/namespaces/{namespace}/podsecuritypolicyselfsubjectreviews",
    "podsecuritypolicysubjectreview": "/oapi/v1/namespaces/{namespace}/podsecuritypolicysubjectreviews",
    "podtemplate": "/api/v1/namespaces/{namespace}/podtemplates",
    "policy": "/oapi/v1/namespaces/{namespace}/policies",
    "policybinding": "/oapi/v1/namespaces/{namespace}/policybindings",
    "replicationcontroller": "/api/v1/namespaces/{namespace}/replicationcontrollers",
    "resourceaccessreview": "/oapi/v1/namespaces/{namespace}/resourceaccessreviews",
    "resourcequota": "/api/v1/namespaces/{namespace}/resourcequotas",
    "role": "/oapi/v1/namespaces/{namespace}/roles",
    "rolebinding": "/oapi/v1/namespaces/{namespace}/rolebindings",
    "rolebindingrestriction": "/oapi/v1/namespaces/{namespace}/rolebindingrestrictions",
    "route": "/oapi/v1/namespaces/{namespace}/routes",
    "secret": "/api/v1/namespaces/{namespace}/secrets",
    "selfsubjectrulesreview": "/oapi/v1/namespaces/{namespace}/selfsubjectrulesreviews",
    "service": "/api/v1/namespaces/{namespace}/services",
    "serviceaccount": "/api/v1/namespaces/{namespace}/serviceaccounts",
    "subjectaccessreview": "/oapi/v1/namespaces/{namespace}/subjectaccessreviews",
    "subjectrulesreview": "/oapi/v1/namespaces/{namespace}/subjectrulesreviews",
    "template": "/oapi/v1/namespaces/{namespace}/templates"
}
CLUSTER_URL = {
    "binding": "/api/v1/bindings",
    "build": "/oapi/v1/builds",
    "buildconfig": "/oapi/v1/buildconfigs",
    "clusternetwork": "/oapi/v1/clusternetworks",
    "clusterpolicy": "/oapi/v1/clusterpolicies",
    "clusterpolicybinding": "/oapi/v1/clusterpolicybindings",
    "clusterresourcequota": "/oapi/v1/clusterresourcequotas",
    "clusterrole": "/oapi/v1/clusterroles",
    "clusterrolebinding": "/oapi/v1/clusterrolebindings",
    "configmap": "/api/v1/configmaps",
    "deploymentconfig": "/oapi/v1/deploymentconfigs",
    "deploymentconfigrollback": "/oapi/v1/deploymentconfigrollbacks",
    "egressnetworkpolicy": "/oapi/v1/egressnetworkpolicies",
    "group": "/oapi/v1/groups",
    "hostsubnet": "/oapi/v1/hostsubnets",
    "identity": "/oapi/v1/identities",
    "image": "/oapi/v1/images",
    "imagesignature": "/oapi/v1/imagesignatures",
    "imagestream": "/oapi/v1/imagestreams",
    "imagestreamimport": "/oapi/v1/imagestreamimports",
    "imagestreammapping": "/oapi/v1/imagestreammappings",
    "imagestreamtag": "/oapi/v1/imagestreamtags",
    "limitrange": "/api/v1/limitranges",
    "localresourceaccessreview": "/oapi/v1/localresourceaccessreviews",
    "localsubjectaccessreview": "/oapi/v1/localsubjectaccessreviews",
    "namespace": "/api/v1/namespaces",
    "netnamespace": "/oapi/v1/netnamespaces",
    "node": "/api/v1/nodes",
    "oauthaccesstoken": "/oapi/v1/oauthaccesstokens",
    "oauthauthorizetoken": "/oapi/v1/oauthauthorizetokens",
    "oauthclient": "/oapi/v1/oauthclients",
    "oauthclientauthorization": "/oapi/v1/oauthclientauthorizations",
    "persistentvolume": "/api/v1/persistentvolumes",
    "persistentvolumeclaim": "/api/v1/persistentvolumeclaims",
    "pod": "/api/v1/pods",
    "podsecuritypolicyreview": "/oapi/v1/podsecuritypolicyreviews",
    "podsecuritypolicyselfsubjectreview": "/oapi/v1/podsecuritypolicyselfsubjectreviews",
    "podsecuritypolicysubjectreview": "/oapi/v1/podsecuritypolicysubjectreviews",
    "podtemplate": "/api/v1/podtemplates",
    "policy": "/oapi/v1/policies",
    "policybinding": "/oapi/v1/policybindings",
    "project": "/oapi/v1/projects",
    "projectrequest": "/oapi/v1/projectrequests",
    "replicationcontroller": "/api/v1/replicationcontrollers",
    "resourceaccessreview": "/oapi/v1/resourceaccessreviews",
    "resourcequota": "/api/v1/resourcequotas",
    "role": "/oapi/v1/roles",
    "rolebinding": "/oapi/v1/rolebindings",
    "rolebindingrestriction": "/oapi/v1/rolebindingrestrictions",
    "route": "/oapi/v1/routes",
    "secret": "/api/v1/secrets",
    "selfsubjectrulesreview": "/oapi/v1/selfsubjectrulesreviews",
    "service": "/api/v1/services",
    "serviceaccount": "/api/v1/serviceaccounts",
    "subjectaccessreview": "/oapi/v1/subjectaccessreviews",
    "subjectrulesreview": "/oapi/v1/subjectrulesreviews",
    "template": "/oapi/v1/templates",
    "user": "/oapi/v1/users",
    "useridentitymapping": "/oapi/v1/useridentitymappings"
}
USER_AGENT = "ansible-openshift-module/0.0.1"


# TODO(erjohnso): SSL Certificate validation is currently unsupported.
# It can be made to work when the following are true:
# - Ansible consistently uses a "match_hostname" that supports IP Address
#   matching. This is now true in >= python3.5.0. Currently, this feature
#   is not yet available in backports.ssl_match_hostname (still 3.4).
# - Ansible allows passing in the self-signed CA cert that is created with
#   a kubernetes master. The lib/ansible/module_utils/urls.py method,
#   SSLValidationHandler.get_ca_certs() needs a way for the OpenShift
#   CA cert to be passed in and included in the generated bundle file.
# When this is fixed, the following changes can be made to this module,
# - Remove the 'return' statement in line 254 below
# - Set 'required=true' for certificate_authority_data and ensure that
#   ansible's SSLValidationHandler.get_ca_certs() can pick up this CA cert
# - Set 'required=true' for the validate_certs param.

def decode_cert_data(module):
    return
    d = module.params.get("certificate_authority_data")
    if d and not d.startswith("-----BEGIN"):
        module.params["certificate_authority_data"] = base64.b64decode(d)


def api_request(module, url, method="GET", headers={}, data=None):
    body = None
    headers.update(base_headers)
    if data:
        data = json.dumps(data)
    response, info = fetch_url(module, url, method=method, headers=headers, data=data)
    if int(info['status']) == -1:
        module.fail_json(msg="Failed to execute the API request: %s" % info['msg'], url=url, method=method, headers=headers)
    if response is not None:
        body = json.loads(response.read())
    return info, body


def openshift_create_resource(module, url, data):
    info, body = api_request(module, url, method="POST", data=data, headers={"Content-Type": "application/json"})
    if info['status'] == 409:
        name = data["metadata"].get("name", None)
        info, body = api_request(module, url + "/" + name)
        return False, body
    elif info['status'] >= 400:
        module.fail_json(msg="failed to create the resource: %s" % info['msg'], url=url)
    return True, body


def openshift_delete_resource(module, url, data):
    name = data.get('metadata', {}).get('name')
    if name is None:
        module.fail_json(msg="Missing a named resource in object metadata when trying to remove a resource")

    url = url + '/' + name
    info, body = api_request(module, url, method="DELETE")
    if info['status'] == 404:
        return False, "Resource name '%s' already absent" % name
    elif info['status'] >= 400:
        module.fail_json(msg="failed to delete the resource '%s': %s" % (name, info['msg']), url=url)
    return True, "Successfully deleted resource name '%s'" % name


def openshift_replace_resource(module, url, data):
    name = data.get('metadata', {}).get('name')
    if name is None:
        module.fail_json(msg="Missing a named resource in object metadata when trying to replace a resource")

    headers = {"Content-Type": "application/json"}
    url = url + '/' + name
    info, body = api_request(module, url, method="PUT", data=data, headers=headers)
    if info['status'] == 409:
        name = data["metadata"].get("name", None)
        info, body = api_request(module, url + "/" + name)
        return False, body
    elif info['status'] >= 400:
        module.fail_json(msg="failed to replace the resource '%s': %s" % (name, info['msg']), url=url)
    return True, body


def openshift_update_resource(module, url, data):
    name = data.get('metadata', {}).get('name')
    if name is None:
        module.fail_json(msg="Missing a named resource in object metadata when trying to update a resource")

    headers = {"Content-Type": "application/strategic-merge-patch+json"}
    url = url + '/' + name
    info, body = api_request(module, url, method="PATCH", data=data, headers=headers)
    if info['status'] == 409:
        name = data["metadata"].get("name", None)
        info, body = api_request(module, url + "/" + name)
        return False, body
    elif info['status'] >= 400:
        module.fail_json(msg="failed to update the resource '%s': %s" % (name, info['msg']), url=url)
    return True, body


base_headers = {}

def main():
    module = AnsibleModule(
        argument_spec=dict(
            http_agent=dict(default=USER_AGENT),

            api_token=dict(aliases=["token"]),
            force_basic_auth=dict(default="yes"),
            validate_certs=dict(default=False, type='bool'),
            certificate_authority_data=dict(required=False),
            api_endpoint=dict(required=True),
            file_reference=dict(required=False),
            inline_data=dict(required=False),
            state=dict(default="present", choices=["present", "absent", "update", "replace"])
        ),
        mutually_exclusive = (('file_reference', 'inline_data')),
        required_one_of = (('file_reference', 'inline_data'),),
    )

    decode_cert_data(module)

    api_endpoint = module.params.get('api_endpoint')
    state = module.params.get('state')
    inline_data = module.params.get('inline_data')
    file_reference = module.params.get('file_reference')
    base_headers['Authorization'] = 'Bearer {0}'.format(module.params.get('api_token'))

    if inline_data:
        if not isinstance(inline_data, dict) and not isinstance(inline_data, list):
            data = yaml.load(inline_data)
        else:
            data = inline_data
    else:
        try:
            f = open(file_reference, "r")
            data = [x for x in yaml.load_all(f)]
            f.close()
            if not data:
                module.fail_json(msg="No valid data could be found.")
        except:
            module.fail_json(msg="The file '%s' was not found or contained invalid YAML/JSON data" % file_reference)

    # set the transport type and build the target endpoint url
    transport = 'https'

    target_endpoint = "%s://%s" % (transport, api_endpoint)

    body = []
    changed = False

    # make sure the data is a list
    if not isinstance(data, list):
        data = [ data ]

    for item in data:
        namespace = None
        if item and 'metadata' in item:
            namespace = item.get('metadata', {}).get('namespace', None)
            kind = item.get('kind', '').lower()
            try:
                if namespace is None:
                    url = target_endpoint + CLUSTER_URL[kind]
                else:
                    url = target_endpoint + KIND_URL[kind]
            except KeyError:
                module.fail_json(msg="invalid resource kind specified in the data: '%s'" % kind)
            url = url.replace("{namespace}", namespace)
        else:
            url = target_endpoint

        if state == 'present':
            item_changed, item_body = openshift_create_resource(module, url, item)
        elif state == 'absent':
            item_changed, item_body = openshift_delete_resource(module, url, item)
        elif state == 'replace':
            item_changed, item_body = openshift_replace_resource(module, url, item)
        elif state == 'update':
            item_changed, item_body = openshift_update_resource(module, url, item)

        changed |= item_changed
        body.append(item_body)

    module.exit_json(changed=changed, api_response=body)


# import module snippets
from ansible.module_utils.basic import *    # NOQA
from ansible.module_utils.urls import *     # NOQA


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import abc
import StringIO
from ansible import errors

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser

from InventoryFilter import InventoryFilter


class DummyInventory(InventoryFilter):

    def get_hostnames(self, topo):
        hostnames = []
        for group in topo['dummy_res']:
            for host in group['hosts']:
                hostnames.append(host)
        return hostnames

    def get_host_ips(self, topo):
        return self.get_hostnames(topo)

    def add_hosts_to_groups(self, config, inven_hosts, layout):
        # create a ip to host mapping based on count
        hosts = {}
        for host_name in layout['hosts']:
            count = layout['hosts'][host_name]['count']
            host_list = []
            hosts[host_name] = [host_list.append(inven_hosts.pop())
                                for inven_hosts in range(count)]
            for ip in host_ips:
                for host_group in layout['hosts'][host_name]['host_groups']:
                    config.set(host_group, ip)
        return config

    def get_inventory(self, topo, layout):
        if len(topo['dummy_res']) == 0:
            return ""
        inventory = ConfigParser(allow_no_value=True)
        layout_hosts = self.get_layout_hosts(layout)
        inven_hosts = self.get_hostnames(topo)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)
        return output.getvalue()



#!/usr/bin/env python

import abc
import StringIO
from ansible import errors

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser

from InventoryFilter import InventoryFilter


class LibvirtInventory(InventoryFilter):

    def get_host_ips(self, topo):
        ips = []
        if not ('libvirt_res' in topo):
            return ips
        for val in topo['libvirt_res']:
            ips.append(val)
        return ips


    def get_inventory(self, topo, layout):

        if len(topo['libvirt_res']) == 0:
            return ""

        inventory = ConfigParser(allow_no_value=True)
        layout_hosts = self.get_layout_hosts(layout)
        inven_hosts = self.get_host_ips(topo)

        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)

        # set children for each host group
        self.set_children(layout)

        # set vars for each host group
        self.set_vars(layout)

        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)

        return output.getvalue()



#!/usr/bin/env python

import abc
import StringIO
from camel import Camel
from ansible import errors

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser

from AWSInventory import AWSInventory
from BeakerInventory import BeakerInventory
from DuffyInventory import DuffyInventory
from DummyInventory import DummyInventory
from GCloudInventory import GCloudInventory
from InventoryFilter import InventoryFilter
from LibvirtInventory import LibvirtInventory
from OpenstackInventory import OpenstackInventory

from InventoryProviders import get_driver, get_all_drivers


class GenericInventory(InventoryFilter):

    def __init__(self):
        InventoryFilter.__init__(self)
        self.filter_classes = get_all_drivers()

    def get_host_ips(self, topo):
        """
        currently it will return the dict as follows:
        {
        "aws_inv": ["x.x.x.x"],
        "os_inv" :["x.x.x.x","x.x.x.x"],
        "gcloud_inv" : ["x.x.x.x","x.x.x.x"]
        }
        """
        host_ip_dict = {}
        for inv_filter in self.filter_classes:
            ips = self.filter_classes[inv_filter]().get_host_ips(topo)
            host_ip_dict[inv_filter] = ips
        return host_ip_dict

    def get_hosts_by_count(self, host_dict, count):
        """
        currently this function gets all the ips/hostname according to the
        order in which inventories are specified. later can be modified
        to work with user input
        """
        all_hosts = []
        for inv in host_dict:
            all_hosts.extend(host_dict[inv])
        return all_hosts[:count]

    def get_inventory(self, topo, layout):
        layout =  Camel().load(layout)
        # get all the topology host_ips
        host_ip_dict = self.get_host_ips(topo)
        # get the count of all layout hosts needed
        layout_host_count = self.get_layout_hosts(layout)
        # generate hosts list based on the layout host count
        inven_hosts = self.get_hosts_by_count(host_ip_dict, layout_host_count)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)
        return output.getvalue()




#!/usr/bin/env python
import os
import sys
import abc
import StringIO
from ansible import errors

def translate_ruletype(ruletype):
    if ruletype == "inbound":
        return "ingress"
    if ruletype == "outbound":
        return "egress"
    else:
        return "invalid ruletype "

class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'os_sg_rule_type': translate_ruletype
        }



try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser

from InventoryFilter import InventoryFilter

class BeakerInventory(InventoryFilter):

    def get_hostnames(self, topology):
        hostnames = []
        if not ('beaker_res' in topology):
            return hostnames
        for group in topology['beaker_res']:
            hostnames.append(group['system'])
        return hostnames

    def get_host_ips(self, topo):
        return self.get_hostnames(topo)

    def add_hosts_to_groups(self, config, inven_hosts, layout):
        pass

    def get_inventory(self, topo, layout):
        if not ('beaker_res' in topo):
            return ''
        if len(topo['beaker_res']) == 0:
            return ''
        inven_hosts = self.get_host_ips(topo)
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)
        return output.getvalue()



#!/usr/bin/env python
import os
import sys
import abc
import StringIO
from ansible import errors

def duplicateattr(output, attr, dattr):
    new_output = []
    for group in output:
        if attr in group:
            new_group = group
            new_group[dattr] = group[attr]
            new_output.append(new_group)
    return output

class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'duplicateattr': duplicateattr
        }



#!/usr/bin/env python

import base64
import optparse
import os
import subprocess
import sys
import traceback
import shutil
try:
    import json
except ImportError:
    import simplejson as json
import ansible.utils.vars as utils_vars
from ansible.parsing.dataloader import DataLoader
from ansible.parsing.utils.jsonify import jsonify
from ansible.parsing.splitter import parse_kv
import ansible.executor.module_common as module_common
import ansible.constants as C


def write_argsfile(argstring, json=False):
    """ Write args to a file for old-style module's use. """
    argspath = os.path.expanduser("~/.ansible_test_module_arguments")
    argsfile = open(argspath, 'w')
    if json:
        args = parse_kv(argstring)
        argstring = jsonify(args)
    argsfile.write(argstring)
    argsfile.close()
    return argspath


def boilerplate_module(modfile, args, interpreter, check, destfile):
    """ simulate what ansible does with new style modules """

    loader = DataLoader()

    complex_args = {}
    if args.startswith("@"):
        # Argument is a YAML file (JSON is a subset of YAML)
        complex_args = utils_vars.combine_vars(complex_args,
                                               loader.load_from_file(args[1:]))
        args = ''
    elif args.startswith("{"):
        # Argument is a YAML document (not a file)
        complex_args = utils_vars.combine_vars(complex_args, loader.load(args))
        args = ''

    if args:
        parsed_args = parse_kv(args)
        complex_args = utils_vars.combine_vars(complex_args, parsed_args)

    task_vars = {}
    if interpreter:
        if '=' not in interpreter:
            print("interpreter must by in the form of \
                   ansible_python_interpreter=/usr/bin/python")
            sys.exit(1)
        interpreter_type, interpreter_path = interpreter.split('=')
        if not interpreter_type.startswith('ansible_'):
            interpreter_type = 'ansible_%s' % interpreter_type
        if not interpreter_type.endswith('_interpreter'):
            interpreter_type = '%s_interpreter' % interpreter_type
        task_vars[interpreter_type] = interpreter_path

    if check:
        complex_args['_ansible_check_mode'] = True

    modname = os.path.basename(modfile)
    modname = os.path.splitext(modname)[0]
    (module_data, module_style, shebang) = module_common.modify_module(
        modname,
        modfile,
        complex_args,
        task_vars=task_vars
    )

    if module_style == 'new' \
       and 'ZIPLOADER_WRAPPER = True' in module_data:
        module_style = 'ziploader'

    modfile2_path = os.path.expanduser(destfile)
    print("* including generated source,\
           if any, saving to: %s" % modfile2_path)
    if module_style not in ('ziploader', 'old'):
        print("* this may offset any line numbers in tracebacks/debuggers!")
    modfile2 = open(modfile2_path, 'w')
    modfile2.write(module_data)
    modfile2.close()
    modfile = modfile2_path

    return (modfile2_path, modname, module_style)


def ziploader_setup(modfile, modname):
    os.system("chmod +x %s" % modfile)

    cmd = subprocess.Popen([modfile, 'explode'],
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE)
    out, err = cmd.communicate()
    lines = out.splitlines()
    if len(lines) != 2 or 'Module expanded into' not in lines[0]:
        print("*" * 35)
        print("INVALID OUTPUT FROM ZIPLOADER MODULE WRAPPER")
        print(out)
        sys.exit(1)
    debug_dir = lines[1].strip()

    argsfile = os.path.join(debug_dir, 'args')
    modfile = os.path.join(debug_dir, 'ansible_module_%s.py' % modname)

    print("* ziploader module detected;\
             extracted module source to: %s" % debug_dir)
    return modfile, argsfile


def runtest(modfile, argspath, modname, module_style):
    """Test run a module, piping it's output for reporting."""
    if module_style == 'ziploader':
        modfile, argspath = ziploader_setup(modfile, modname)

    os.system("chmod +x %s" % modfile)

    invoke = "%s" % (modfile)
    if argspath is not None:
        invoke = "%s %s" % (modfile, argspath)

    cmd = subprocess.Popen(invoke,
                           shell=True,
                           stdout=subprocess.PIPE,
                           stderr=subprocess.PIPE)
    (out, err) = cmd.communicate()

    try:
        results = json.loads(out)
    except:
        print("*" * 35)
        print("INVALID OUTPUT FORMAT")
        print(out)
        traceback.print_exc()
        sys.exit(1)

    print("*" * 35)
    print("PARSED OUTPUT")
    print(jsonify(results, format=True))
    return results


def rundebug(debugger, modfile, argspath, modname, module_style):
    """Run interactively with console debugger."""

    if module_style == 'ziploader':
        modfile, argspath = ziploader_setup(modfile, modname)

    if argspath is not None:
        subprocess.call("%s %s %s" % (debugger, modfile, argspath), shell=True)
    else:
        subprocess.call("%s %s" % (debugger, modfile), shell=True)


def run_module(options):
    m_path = options["module_path"]
    m_args = options["module_args"]
    m_intr = options["interpretor"]
    m_chk = options["check"]
    m_fname = options["filename"]
    (modfile, modname, module_style) = boilerplate_module(m_path,
                                                          m_args,
                                                          m_intr,
                                                          m_chk,
                                                          m_fname)

    argspath = None
    if module_style not in ('new', 'ziploader'):
        if module_style == 'non_native_want_json':
            argspath = write_argsfile(options.module_args, json=True)
        elif module_style == 'old':
            argspath = write_argsfile(options.module_args, json=False)
        else:
            raise Exception("internal error,\
                            unexpected module style: %s" % module_style)
    results = runtest(modfile, argspath, modname, module_style)
    return results



"""
Inventory Providers List
"""

from AWSInventory import AWSInventory
from BeakerInventory import BeakerInventory
from DuffyInventory import DuffyInventory
from DummyInventory import DummyInventory
from GCloudInventory import GCloudInventory
from InventoryFilter import InventoryFilter
from LibvirtInventory import LibvirtInventory
from OpenstackInventory import OpenstackInventory

filter_classes = {
           "aws_inv": AWSInventory,
           "beaker_inv": BeakerInventory,
           "duffy_inv": DuffyInventory,
           "dummy_inv": DummyInventory,
           "gcloud_inv": GCloudInventory,
           "libvirt_inv": LibvirtInventory,
           "os_inv": OpenstackInventory,
}


def get_driver(provider):
    try:
        filter_class = filter_classes[provider]
    except KeyError:
        print("key not found in dictionary")
    return filter_classes[provider]


def get_all_drivers():
    return filter_classes



from action_manager import ActionManager
from Naked.toolshed.shell import run_rb
from cerberus import Validator

from linchpin.exceptions import HookError


class RubyActionManager(ActionManager):

    def __init__(self, name, action_data, target_data, **kwargs):
        
        """
        RubyActionManager constructor
        :param name: Name of Action Manager , ( ie., ruby)
        :param action_data: dictionary of action_block consists of set of actions
        example:
        - name: nameofhook
          type: ruby
          context: true
          actions:
            - test.js
        :param target_data: Target specific data defined in PinFile
        :param kwargs: anyother keyword args passed as metadata
        """

        self.name = name
        self.action_data = action_data
        self.target_data = target_data
        self.context = kwargs.get('context', True)
        self.kwargs = kwargs

    def validate(self):

        """
        Validates the action_block based on the cerberus schema
        """

        schema= {
        'name': {'type': 'string', 'required': True },
        'type': { 'type': 'string', 'allowed': ['ruby']},
        'path': {'type': 'string', 'required': False},
        'context': {'type': 'boolean', 'required': False},
        'actions': {
                     'type': 'list',
                     'schema': {'type':'string'},
                     'required': True
                   }
        }
        v = Validator(schema)
        status = v.validate(self.action_data)
        if not status:
            raise HookError("Invalid syntax: LinchpinHook:"+str((v.errors)))
        else:
            return status
        pass

    def add_ctx_params(self, file_path, context=True):

        """
        Adds ctx params to the action_block run when context is true
        :param file_path: path to the script
        :param context: whether the context params are to be included or not
        """

        if not context:
            return file_path
        params = file_path
        for key in self.target_data:
            params += " %s=%s " %(key, self.target_data[key])
        return "{0} {1}".format(file_path,
                                    params)

    def execute(self):
        
        """
        Executes the action_block in the PinFile
        """

        for action in self.action_data["actions"]:
            context = self.action_data.get("context", True)
            path = self.action_data["path"]
            file_path = "{0}/{1}".format(
                        path,
                        action
                        )
            command = self.add_ctx_params(file_path, context)
            success = run_rb(command)
            return success



#!/usr/bin/env python
import os
import xml.etree.ElementTree as eT

from bkr.client import conf, BeakerCommand
from bkr.common.hub import HubProxy
from bkr.common.pyconfig import PyConfigParser
from json import dumps, loads
from time import sleep
from sys import stderr


BEAKER_CONF = \
    (os.environ.get('BEAKER_CONF', '/etc/beaker/client.conf'))
WAIT_TIME = 60


class BeakerTargets(object):
    def __init__(self, params={}, logger=None):
        self.__dict__ = params.copy()
        self.conf = PyConfigParser()
        default_config = os.path.expanduser(BEAKER_CONF)
        self.conf.load_from_file(default_config)
        self.hub = HubProxy(logger=logger, conf=self.conf)

    def _get_url(self, bkr_id):
        """
        Constructs the Beaker URL for the job related to the provided Beaker
        ID. That ID should be all numeric, unless the structure of Beaker
        changes in the future. If that's the case, then the ID should be
        appropriately URL encoded to be appended to the end of a URL properly.
        """
        base = self.conf.get('HUB_URL', '')
        if base == '':
            raise Exception("Unable to construct URL")
        if base[-1] != '/':
            base += '/'
        return base + 'jobs/' + bkr_id

    def get_system_statuses(self):
        """
        Checks on the status of a set of Beaker jobs (ids) and returns their
        hostname once the jobs have reached their defined status.
        """
        attempts = 0
        pass_count = 0
        all_count = len(self.ids)
        while attempts < self.max_attempts:
            job_results = self._check_jobs(self.ids)
            pass_count = 0
            for resource in job_results['resources']:
                result = resource['result']
                status = resource['status']
                print >> stderr, "status: %s, result: %s" % (status, result)
                if status not in ['Cancelled', 'Aborted']:
                    if result == 'Pass' or (result == 'Warn' and self.skip_no_system):
                        pass_count += 1
                    elif result in ['Fail', 'Warn', 'Panic', 'Completed']:
                        raise Exception("System failed with state '{0}'"\
                                .format(result))
                elif status == 'Aborted':
                    if result == 'Warn' and self.skip_no_system:
                        pass_count += 1
                    else:
                        raise Exception("System aborted")
                elif status == 'Cancelled':
                    raise Exception("System canceled")
            attempts += 1
            if pass_count == all_count:
                return job_results['resources']
            sleep(WAIT_TIME)
        raise Exception("{0} system(s) never completed in {1} polling attempts. {2}"\
                .format(all_count - pass_count, attempts, dumps(job_results)))

    def _check_jobs(self, ids):
        """
            Get state of a job in Beaker
        """
        jobs = ["J:" + _id for _id in ids]
        results = {}
        resources = []
        bkrcmd = BeakerCommand('BeakerCommand')
        bkrcmd.check_taskspec_args(jobs)
        for task in jobs:
            myxml = self.hub.taskactions.to_xml(task)
            myxml = myxml.encode('utf8')
            root = eT.fromstring(myxml)
            # TODO: Using getiterator() since its backward compatible
            # with Python 2.6
            # This is deprectated in 2.7 and we should be using iter()
            for job in root.getiterator('job'):
                results.update({'job_id': job.get('id'),
                                'results': job.get('result')})
            for recipe in root.getiterator('recipe'):
                resources.append({'family': recipe.get('family'),
                                  'distro': recipe.get('distro'),
                                  'arch': recipe.get('arch'),
                                  'variant': recipe.get('variant'),
                                  'system': recipe.get('system'),
                                  'status': recipe.get('status'),
                                  'result': recipe.get('result'),
                                  'id': recipe.get('job_id')})
                results.update({'resources': resources})
        return results


def main():
    mod = AnsibleModule(argument_spec={
        'ids': {'type': 'list'},
        'skip_no_system': {'type': 'bool', 'default': False},
        'max_attempts': {'type': 'int', 'default': 60}
    })
    beaker = BeakerTargets(mod.params)
    try:
        results=beaker.get_system_statuses()
        mod.exit_json(hosts=results, changed=True, success=True)
    except Exception as ex:
        mod.fail_json(msg=str(ex))


# import module snippets
from ansible.module_utils.basic import *
main()



import ansible
from ansible.plugins.callback import CallbackBase


class PlaybookCallback(CallbackBase):

    """Playbook callback"""

    def __init__(self):
        super(PlaybookCallback, self).__init__()
        # store all results
        self.results = []


    def v2_runner_on_ok(self, result):

        """Save ok result"""

        self.results.append(result)

    def v2_runner_on_failed(self, result, **kwargs):

        """Save failed result"""

        self.results.append(result)



import os

import tempfile
import ConfigParser
from collections import OrderedDict

from linchpin.api import LinchpinError, LinchpinAPI

"""
Provide valid API data to test against.

"""

class APIData(object):

    def __init__(self):

        pass




from subprocess_action_manager import SubprocessActionManager
from ansible_action_manager import AnsibleActionManager
from python_action_manager import PythonActionManager
from ruby_action_manager import RubyActionManager
from nodejs_action_manager import NodejsActionManager

# dict of core action manager packaged with linchpin
ACTION_MANAGERS = {
    "shell" : SubprocessActionManager,
    "ansible" : AnsibleActionManager,
    "python": PythonActionManager,
    "ruby" : RubyActionManager,
    "nodejs" : NodejsActionManager
}



import os

import tempfile
import ConfigParser
from collections import OrderedDict

from linchpin.api import LinchpinError

"""
Provide valid context data to test against.

"""

class ContextData(object):

    def __init__(self, parser=ConfigParser.SafeConfigParser):


        self.lib_path = '{0}'.format(os.path.dirname(
            os.path.realpath(__file__))).rstrip('/')

        self.logfile = tempfile.mktemp(suffix='.log', prefix='linchpin')
        self.parser = parser()


    def load_config_data(self, provider='dummy'):
        """
        Load a test-based linchpin.conf into both a configs and evars
        dictionary to represent a configuration file

        """

        self.cfg_data = {}

        expanded_path = None
        config_found = False

        # simply modify this variable to adjust where linchpin.conf can be found
        CONFIG_PATH = [
            '{0}/{1}/conf/linchpin.conf'.format(self.lib_path, provider)
        ]

        for path in CONFIG_PATH:
            expanded_path = (
                "{0}".format(os.path.realpath(os.path.expanduser(path))))

            # implement first found
            if os.path.exists(expanded_path):
                config_found = True
                break

        if not config_found:
            raise LinchpinError('Configuration file not found in'
                                ' path: {0}'.format(CONFIG_PATH))

        config = ConfigParser.SafeConfigParser()
        try:
            f = open(path)
            config.readfp(f)
            f.close()
        except ConfigParser.InterpolationSyntaxError as e:
            raise LinchpinError('Unable to parse configuration file properly:'
                    ' {0}'.format(e))

        for section in config.sections():
            if section not in self.cfg_data:
                self.cfg_data[section] = {}

            # add evars to the ansible extra_vars when running a playbook.
            for k, v in config.items(section):
                if section != 'evars':
                    self.cfg_data[section][k] = v
                else:
                    try:
                        self.cfg_data[section][k] = config.getboolean(section, k)
                    except ValueError as e:
                        self.cfg_data[section][k] = v

        # override logger file
        self.cfg_data['logger']['file'] = self.logfile

        self.evars = self.cfg_data.get('evars', {})


    def get_temp_filename(self):

        tmpfile = tempfile.NamedTemporaryFile(delete=False).name
        return tmpfile


    def write_config_file(self, path):

        try:
            with open(path, 'a+') as f:
                self.parser.write(f)
        except Exception as e:
            raise LinchpinError('Unable to write configuration file:'
                    ' {0}'.format(e))


    def parse_config(self, config_data=None):

        """
        Creates a config object using ConfigParser from the config_data object

        """

        if not config_data:
            config_data = self.cfg_data

        #we know that data is a dict, containing dicts
        for k, v in config_data.iteritems():
            self.parser.add_section(k)
            for kv, vv in v.iteritems():
                if type(vv) is not str:
                    vv = str(vv)
                self.parser.set(k, kv, vv)




#!/usr/bin/env python
import yaml
import json


def format_output(output):
    output = json.dumps(output)
    return output

class FilterModule(object):
    ''' A filter to fix output format '''
    def filters(self):
        return {
            'unicode_filter': format_output
        }



import yaml
import json
import jsonschema
from jsonschema import validate


def get_cred_schema(cred_type):
    aws_cred_schema = {
        "type": "object",
        "properties": {
           "aws_access_key_id": {"type": "string"},
           "aws_secret_access_key": {"type": "string"},
        },
        "required": ["aws_access_key_id", "aws_secret_access_key"]
    }
    os_cred_schema = {
        "type": "object",
        "properties": {
           "endpoint": {"type": "string"},
           "project": {"type": "string"},
           "username": {"type": "string"},
           "password": {"type": "string"},
        },
        "required": ["endpoint", "project", "username", "password"]
    }
    gcloud_cred_schema = {
        "type": "object",
        "properties": {
           "service_account_email": {"type": "string"},
           "project_id": {"type": "string"},
           "credentials_file": {"type": "string"},
        },
        "required": ["service_account_email", "project_id", "credentials_file"]
    }
    duffy_cred_schema = {
        "type": "object",
        "properties": {
           "key_path": {"type": "string"},
           "url_base": {"type": "string"},
        },
    }
    rx_cred_schema = {
        "type": "object",
        "properties": {
           "username": {"type": "string"},
           "api_key": {"type": "string"},
        },
    }
    cred_schemas = {
       "aws": aws_cred_schema,
       "os": os_cred_schema,
       "gcloud": gcloud_cred_schema,
       "duffy": duffy_cred_schema,
       "rax": rx_cred_schema
    }
    return cred_schemas[cred_type]


def validate_creds(cred_path, cred_type):
    schema = get_cred_schema(cred_type)
    cred_str = open(cred_path).read()
    cred_json = yaml.load(cred_str)
    try:
        validate(cred_json, schema)
        return True
    except:
        return False



#!/usr/bin/env python

import os
import sys
import ast
import shutil
import logging

from distutils import dir_util
from collections import OrderedDict
from jinja2 import Environment, PackageLoader

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser as ConfigParser

from linchpin.api.context import LinchpinContext
from linchpin.exceptions import LinchpinError
from linchpin.cli import LinchpinCli
from linchpin.version import __version__


class LinchpinCliContext(LinchpinContext):
    """
    LPContext object, which will be used to manage the cli,
    and load the configuration file.
    """


    def __init__(self):
        """
        Initializes basic variables
        """

        # The following values are set in the parent class
        #
        # self.version = __version__
        # self.verbose = False
        #
        # lib_path = '{0}'.format(os.path.dirname(
        #                       os.path.realpath(__file__))).rstrip('/')
        # self.lib_path = os.path.realpath(os.path.join(lib_path, os.pardir))
        #
        #self.workspace = os.path.realpath(os.path.curdir)

        LinchpinContext.__init__(self)


    def load_config(self, lpconfig=None):
        """
        Create self.cfgs from the linchpin configuration file.
        .. note:: Overrides load_config in linchpin.api.LinchpinContext

        These are the only hardcoded values, which are used to find the config
        file. The search path, is a first found of the following::

          * $PWD/linchpin.conf
          * ~/.linchpin.conf
          * /etc/linchpin.conf
          * /linchpin/library/path/linchpin.conf

        Alternatively, a full path to the linchpin configuration file
        can be passed.

        :param lpconfig: absolute path to a linchpin config (default: None)

        """

        self.cfgs = {}

        expanded_path = None
        config_found = False

        if lpconfig:
            CONFIG_PATH = [ lpconfig ]
        else:
            # simply modify this variable to adjust where linchpin.conf can be found
            CONFIG_PATH = [
                '{0}/linchpin.conf'.format(
                            os.path.realpath(os.path.expanduser(os.path.curdir))),
                '~/.linchpin.conf',
                '/etc/linchpin.conf',
                '{0}/linchpin.conf'.format(self.lib_path)
            ]

        for path in CONFIG_PATH:
            expanded_path = (
                "{0}".format(os.path.realpath(os.path.expanduser(path))))

            # implement first found
            if os.path.exists(expanded_path):
                # logging before the config file is setup doesn't work
                # if messages are needed before this, use print.
                config_found = True
                break

        if not config_found:
            raise LinchpinError('Configuration file not found in'
                                ' path: {0}'.format(CONFIG_PATH))

        config = ConfigParser.SafeConfigParser()
        try:
            f = open(path)
            config.readfp(f)
            f.close()
        except ConfigParser.InterpolationSyntaxError as e:
            raise LinchpinError('Unable to parse configuration file properly:'
                    ' {0}'.format(e))

        for section in config.sections():
            if section not in self.cfgs:
                self.cfgs[section] = {}

            # add evars to the ansible extra_vars when running a playbook.
            for k, v in config.items(section):
                if section != 'evars':
                    self.cfgs[section][k] = v
                else:
                    try:
                        self.cfgs[section][k] = config.getboolean(section, k)
                    except ValueError as e:
                        self.cfgs[section][k] = v


    def setup_logging(self):

        """
        Setup logging to a file, console, or both.  Modifying the linchpin.conf
        appropriately will provide functionality.

        """

        self.enable_logging = ast.literal_eval(self.cfgs['logger'].get('enable', 'True'))

        if self.enable_logging:

            # create logger
            self.logger = logging.getLogger('lp_logger')
            self.logger.setLevel(eval(self.cfgs['logger'].get('level',
                                                            'logging.DEBUG')))

            fh = logging.FileHandler(self.cfgs['logger'].get('file',
                                                            'linchpin.log'))
            fh.setLevel(eval(self.cfgs['logger'].get('level',
                                                    'logging.DEBUG')))
            formatter = logging.Formatter(
                            self.cfgs['logger'].get('format',
                            '%(levelname)s %(asctime)s %(message)s'))
            fh.setFormatter(formatter)
            self.logger.addHandler(fh)


        self.console = logging.getLogger('lp_console')
        self.console.setLevel(eval(self.cfgs['console'].get('level',
                                                        'logging.INFO')))

        ch = logging.StreamHandler()
        ch.setLevel(eval(self.cfgs['console'].get('level',
                                                'logging.INFO')))
        formatter = logging.Formatter(
                        self.cfgs['console'].get('format', '%(message)s'))
        ch.setFormatter(formatter)
        self.console.addHandler(ch)


    def log(self, msg, **kwargs):
        """
        Logs a message to a logfile or the console

        :param msg: message to log

        :param lvl: keyword argument defining the log level

        :param msg_type: keyword argument giving more flexibility.
        Only `STATE` is currently implemented.
        """

        lvl = kwargs.get('level')
        msg_type = kwargs.get('msg_type')

        if lvl is None:
            lvl = logging.INFO

        if self.verbose and not msg_type:
            self.console.log(logging.INFO, msg)

        state_msg = msg
        if msg_type == 'STATE':
            state_msg = 'STATE - {0}'.format(msg)
            self.console.log(logging.INFO, msg)

        if self.enable_logging:
            self.logger.log(lvl, state_msg)


    def log_state(self, msg):
        """Logs a message to stdout."""

        self.log(msg, msg_type='STATE', level=logging.DEBUG)

    def log_info(self, msg):
        """Logs a message to """
        self.log(msg, level=logging.INFO)

    def log_debug(self, msg):
        """Logs a message to stderr."""
        self.log(msg, level=logging.DEBUG)




#!/usr/bin/env python

import os
import sys
import click
import shutil
import logging

from distutils import dir_util
from jinja2 import Environment, PackageLoader

from linchpin.cli import LinchpinCli
from linchpin.version import __version__
from linchpin.exceptions import *
from linchpin.cli.context import LinchpinCliContext


pass_context = click.make_pass_decorator(LinchpinCliContext, ensure=True)
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


class LinchpinAliases(click.Group):

    lp_commands = ['init', 'up', 'destroy']
    lp_aliases = {
            'rise': 'up',
            'drop': 'destroy',
            'down': 'destroy',
        }

    def list_commands(self, ctx):
        """
        Provide a list of available commands. Anhthing deprecated should
        not be listed
        """

        return self.lp_commands

    def get_command(self, ctx, name):

        """
        Track aliases for specific commands and the commands and return the
        correct action.
        """

        cmd = self.lp_aliases.get(name)

        if cmd == None:
            cmd = name

        rv = click.Group.get_command(self, ctx, cmd)
        return rv


def _handle_results(ctx, results):
    """
    Handle results from the Ansible API. Either as a return value (retval)
    when running with the ansible console enabled, or as a list of TaskResult
    objects, and a return value.

    If a target fails along the way, this method immediately exits with the
    appropriate return value (retval). If the ansible console is disabled, an
    error message will be printed before exiting.

    :param results:
        The dictionary of results for each target.
    """

    retval = 0

    for k,v in results.iteritems():
        if not isinstance(v, int):
            trs = v

            if trs is not None:
                trs.reverse()
                tr = trs[0]
                if tr.is_failed():
                    msg = tr._check_key('msg')
                    ctx.log_state("Target '{0}': {1} failed with error '{2}'".format(
                                                            k,
                                                            tr._task,
                                                            msg))
                    sys.exit(retval)
        else:
            if v: sys.exit(v)


@click.command(cls=LinchpinAliases,
                invoke_without_command=True,
                no_args_is_help=True,
                context_settings=CONTEXT_SETTINGS)
@click.option('-c', '--config', type=click.Path(), envvar='LP_CONFIG',
        help='Path to config file')
@click.option('-w', '--workspace', type=click.Path(), envvar='WORKSPACE',
        help='Use the specified workspace if the familiar Jenkins $WORKSPACE environment variable '
        'is not set')
@click.option('-v', '--verbose', is_flag=True, default=False,
        help='Enable verbose output')
@click.option('--version', is_flag=True,
        help='Prints the version and exits')
@click.option('--creds-path', type=click.Path(), envvar='LP_CREDS',
        help='Use the specified credentials path if WORKSPACE environment variable '
        'is not set')
@pass_context
def runcli(ctx, config, workspace, verbose, version, creds_path):
    """linchpin: hybrid cloud orchestration"""

    ctx.verbose = verbose

    ctx.load_config(lpconfig=config)
    ctx.load_global_evars()
    ctx.setup_logging()

    if version:
        ctx.log_state('linchpin version {0}'.format(ctx.version))
        sys.exit(0)

    if workspace is not None:
        ctx.workspace = os.path.realpath(os.path.expanduser(workspace))
    else:
        ctx.workspace = os.getenv('PWD')

    if creds_path is not None:
        ctx.set_evar('creds_path', os.path.realpath(os.path.expanduser(creds_path)))

    ctx.log_debug("ctx.workspace: {0}".format(ctx.workspace))

    ctx.pinfile = ctx.get_cfg('init', 'pinfile', default='PinFile')


@runcli.command('init', short_help='Initializes a linchpin project.')
@pass_context
def init(ctx):
    """
    Initializes a linchpin project, which generates an example PinFile, and
    creates the necessary directory structure for topologies and layouts.

    :param ctx: Context object defined by the click.make_pass_decorator method
    """

    ws = ctx.workspace
    pf = ctx.pinfile

    pf_w_path = os.path.realpath('{0}/{1}'.format(ws, pf))

    src = ctx.get_cfg('init', 'source', 'templates/')
    src_w_path = os.path.realpath('{0}/{1}'.format(ctx.lib_path, src))

    src_pf = os.path.realpath('{0}.lp_example'.format(pf_w_path))

    try:

        if os.path.exists(pf_w_path):
            if not click.confirm(
                '{0} already exists, overwrite it?'.format(pf_w_path),
                default=False):
                sys.exit(0)

        dir_util.copy_tree(src_w_path, ws, verbose=1)
        os.rename(src_pf, pf_w_path)

        ctx.log_state('{0} and file structure created at {1}'.format(
            ctx.pinfile, ctx.workspace))

    except Exception as e:
        ctx.log_state('Error: {0}'.format(e))
        sys.exit(1)



@runcli.command()
@click.option('-p', '--pinfile', envvar='PINFILE',
        help='Use a different PinFile than the one in the current workspace.')
@click.argument('targets', metavar='TARGETS', required=False,
        nargs=-1)
@pass_context
def up(ctx, pinfile, targets):
    """
    Provisions nodes from the given target(s) in the given PinFile.

    :param ctx: Context object defined by the click.make_pass_decorator method

    :param pinfile:
        path to pinfile (Default: ctx.workspace)

    :param targets:
        Provision ONLY the listed target(s). If omitted, ALL targets in the
        appropriate PinFile will be provisioned.
    """

    if pinfile is None:
        pinfile = ctx.pinfile

    pf_w_path = '{0}/{1}'.format(ctx.workspace, pinfile)

    if not os.path.exists(pf_w_path):
        ctx.log_state('{0} not found in provided workspace: '
            '{1}'.format(pinfile, ctx.workspace))
        sys.exit(1)

    lpcli = LinchpinCli(ctx)
    try:
        results = lpcli.lp_up(pf_w_path, targets)

        _handle_results(ctx, results)

    except LinchpinError as e:
        ctx.log_state(e)
        sys.exit(1)


@runcli.command()
@click.option('-p', '--pinfile', envvar='PINFILE',
        help='Use a name for the PinFile different from the configuration.')
@click.argument('targets', metavar='TARGET', required=False,
        nargs=-1)
@pass_context
def rise(ctx, pinfile, targets):
    """
    DEPRECATED. Use 'up'

    """

    pass


@runcli.command()
@click.option('-p', '--pinfile', envvar='PINFILE',
        help='Use a different PinFile than the one in the current workspace.')
@click.argument('targets', metavar='TARGET', required=False,
        nargs=-1)
@pass_context
def destroy(ctx, pinfile, targets):
    """
    Destroys nodes from the given target(s) in the given PinFile.

    :param ctx: Context object defined by the click.make_pass_decorator method

    :param pinfile:
        path to pinfile (Default: ctx.workspace)

    :param targets:
        Destroy ONLY the listed target(s). If omitted, ALL targets in the
        appropriate PinFile will be destroyed.

    """

    if pinfile is None:
        pinfile = ctx.pinfile

    pf_w_path = '{0}/{1}'.format(ctx.workspace, pinfile)

    if not os.path.exists(pf_w_path):
        return ctx.log_state('{0} not found in provided workspace: '
            '{1}'.format(pinfile, ctx.workspace))

    lpcli = LinchpinCli(ctx)
    try:
        results = lpcli.lp_destroy(pf_w_path, targets)

        _handle_results(ctx, results)

    except LinchpinError as e:
        ctx.log_state(e)
        sys.exit(1)


@runcli.command()
@click.option('-p', '--pinfile', envvar='PINFILE',
        help='Use a name for the PinFile different from the configuration.')
@click.argument('targets', metavar='TARGET', required=False,
        nargs=-1)
@pass_context
def drop(ctx, pinfile, targets):
    """
    DEPRECATED. Use 'destroy'.

    There are now two functions, `destroy` and `down` which perform node
    teardown. The `destroy` functionality is the default, and if drop is
    used, will be called.

    The `down` functionality is currently unimplemented, but will shutdown
    and preserve instances. This feature will only work on providers that
    support this option.

    """

    pass





#@cli.group()
#@pass_config
#def topology(config):
#    pass
#
#
#@topology.command(name='list')
#@click.option('--upstream',
#              default=None,
#              required=False,
#              help="upstream url for topology")
#@pass_config
#def topology_list(config, upstream):
#    lpcli = LinchpinCli()
#    click.echo(": TOPOLOGIES LIST :")
#    files = lpcli.lp_topo_list(upstream)
#    t_files = []
#    for i in range(0, len(files)):
#        t_files.append((i+1, files[i]["name"]))
#    headers = ["Sno", "Name"]
#    print tabulate(t_files, headers, tablefmt="fancy_grid")
#
#
#@topology.command(name='get')
#@click.option('--upstream',
#              default=None,
#              required=False,
#              help="upstream url for topology")
#@click.argument('topo')
#@pass_config
#def topology_get(config, topo, upstream):
#    """
#    needs implementation if topology folder is not found raise error
#    """
#    lpcli = LinchpinCli()
#    d = lpcli.lp_topo_get(topo)
#    pprint.pprint(d)
#
#
#@cli.group()
#@pass_config
#def layout(config):
#    pass
#
#
#@layout.command(name='list')
#@click.option('--upstream',
#              default=None,
#              required=False,
#              help="upstream url for layouts")
#@pass_config
#def layouts_list(config, upstream):
#    lpcli = LinchpinCli()
#    click.echo(": LAYOUTS LIST :")
#    files = lpcli.lp_layout_list(upstream)
#    t_files = []
#    for i in range(0, len(files)):
#        t_files.append((i+1, files[i]["name"]))
#    headers = ["Sno", "Name"]
#    print tabulate(t_files, headers, tablefmt="fancy_grid")
#
#
#@layout.command(name='get')
#@click.option('--upstream',
#              default=None,
#              required=False,
#              help="upstream url for layouts")
#@click.argument('layout')
#@pass_config
#def layouts_get(config, layout, upstream):
#    """
#    needs implementation if layout folder is not found raise error
#    """
#    lpcli = LinchpinCli()
#    output = lpcli.lp_layout_get(layout, upstream)
#    pprint.pprint(output)
#
#
#@cli.command()
#@click.option("--pf",
#              default=False,
#              required=False,
#              help="gets the topology by name")
#@click.option("--target",
#              default="all",
#              required=False,
#              help="target name mentioned in PinFile")
#@pass_config
#def rise(config, pf, target):
#    """ rise module of linchpin cli """
#    init_dir = os.getcwd()
#    pfs = list_by_ext(init_dir, "PinFile")
#    if len(pfs) == 0:
#        display("ERROR:001")
#    if len(pfs) > 1:
#        display("ERROR:002")
#    pf = pfs[0]
#    lpcli = LinchpinCli()
#    output = lpcli.lp_rise(pf, target)
#
#
#
#@cli.command()
#@click.option("--pf",
#              default=False,
#              required=False,
#              help="gets the PinFile by name")
#@click.option("--layout",
#              default=False,
#              required=False,
#              help="gets the layout by name")
#@click.option("--topo",
#              default=False,
#              required=False,
#              help="gets the topology by name")
#@pass_config
#def validate(config, topo, layout, pf):
#    """ validate module of linchpin cli : currenly validates only topologies,
#        need to implement PinFile, layouts too"""
#    lpcli = LinchpinCli()
#    topo = os.path.abspath(topo)
#    output = lpcli.lp_validate(topo, layout, pf)
#    pprint.pprint(output)
#
#
#@cli.command()
#@click.option("--init",
#              default=False,
#              required=False,
#              is_flag=True,
#              help="Initialises config file")
#@click.option("--reset",
#              default=False,
#              required=False,
#              is_flag=True,
#              help="sets existing config file parameters")
#@pass_config
#def config(config, reset, init):
#    """ config module of linchpin cli"""
#    if reset:
#        if os.path.isfile("./linchpin_config.yml"):
#            display("WARNING:002", "prompt")
#        config.lpconfig.stream(playbook_dir=config.clipath,
#                               pwd=os.getcwd()).dump('linchpin_config.yml')
#    if init:
#        if not os.path.isfile("./linchpin_config.yml"):
#            display("ERROR:004", "print")
#        conf = yaml.load(open("linchpin_config.yml", "r").read())
#        for key in conf:
#            inp_str = raw_input("Enter value of "+key+":("+str(conf[key])+"):")
#            if inp_str != "":
#                conf[key] = inp_str
#        config.lpconfig.stream(
#                         playbook_dir=config.clipath,
#                         keystore_path=conf["keystore_path"],
#                         outputfolder_path=conf["outputfolder_path"],
#                         inventoryfolder_path=conf["inventoryfolder_path"],
#                         async=conf["async"],
#                         async_timeout=conf["async_timeout"],
#                         no_output=conf["no_output"],
#                         schema=conf["schema"],
#                         default_layouts_path=conf["default_layouts_path"],
#                         inventory_outputs_path=conf["inventory_outputs_path"],
#                         check_mode=conf["check_mode"],
#                         pwd=os.getcwd()).dump('linchpin_config.yml')
#
#
#@cli.command()
#@click.option("--invtype", default="generic", required=False,
#              type=click.Path(),
#              help="inventory type")
#@click.option("--invout", required=True, type=click.Path(),
#              help="inventory output file")
#@click.option("--layout", default=False, required=True,  type=click.Path(),
#              help="layout file usually found in layout folder")
#@click.option("--topoout", default=False, required=True, type=click.Path(),
#              help="topology output file usually found in output folders")
#@pass_config
#def invgen(config, topoout, layout, invout, invtype):
#    """ invgen module of linchpin cli """
#    topoout = os.path.abspath(topoout)
#    layout = os.path.abspath(layout)
#    invout = os.path.abspath(invout)
#    lpcli = LinchpinCli()
#    result = lpcli.lp_invgen(topoout, layout, invout, invtype)
#    pprint.pprint(result)

def main():
    print("entrypoint")



#!/usr/bin/python
# coding: utf-8 -*-

# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
# Copyright (c) 2013, Benno Joy <benno@ansible.com>
# Copyright (c) 2013, John Dewey <john@dewey.ws>
#
# This module is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.


try:
    import shade
    from shade import meta
    HAS_SHADE = True
except ImportError:
    HAS_SHADE = False


ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'version': '1.0'}

DOCUMENTATION = '''
---
module: os_server
short_description: Create/Delete Compute Instances from OpenStack
extends_documentation_fragment: openstack
version_added: "2.0"
author: "Monty Taylor (@emonty)"
description:
   - Create or Remove compute instances from OpenStack.
options:
   name:
     description:
        - Name that has to be given to the instance
     required: true
   image:
     description:
        - The name or id of the base image to boot.
     required: true
   image_exclude:
     description:
        - Text to use to filter image names, for the case, such as HP, where
          there are multiple image names matching the common identifying
          portions. image_exclude is a negative match filter - it is text that
          may not exist in the image name. Defaults to "(deprecated)"
   flavor:
     description:
        - The name or id of the flavor in which the new instance has to be
          created. Mutually exclusive with flavor_ram
     required: false
     default: 1
   flavor_ram:
     description:
        - The minimum amount of ram in MB that the flavor in which the new
          instance has to be created must have. Mutually exclusive with flavor.
     required: false
     default: 1
   flavor_include:
     description:
        - Text to use to filter flavor names, for the case, such as Rackspace,
          where there are multiple flavors that have the same ram count.
          flavor_include is a positive match filter - it must exist in the
          flavor name.
   key_name:
     description:
        - The key pair name to be used when creating a instance
     required: false
     default: None
   security_groups:
     description:
        - Names of the security groups to which the instance should be
          added. This may be a YAML list or a comma separated string.
     required: false
     default: None
   network:
     description:
        - Name or ID of a network to attach this instance to. A simpler
          version of the nics parameter, only one of network or nics should
          be supplied.
     required: false
     default: None
   nics:
     description:
        - A list of networks to which the instance's interface should
          be attached. Networks may be referenced by net-id/net-name/port-id
          or port-name.
        - 'Also this accepts a string containing a list of (net/port)-(id/name)
          Eg: nics: "net-id=uuid-1,port-name=myport"
          Only one of network or nics should be supplied.'
     required: false
     default: None
   auto_ip:
     description:
        - Ensure instance has public ip however the cloud wants to do that
     required: false
     default: 'yes'
     aliases: ['auto_floating_ip', 'public_ip']
   floating_ips:
     description:
        - list of valid floating IPs that pre-exist to assign to this node
     required: false
     default: None
   floating_ip_pools:
     description:
        - Name of floating IP pool from which to choose a floating IP
     required: false
     default: None
   meta:
     description:
        - 'A list of key value pairs that should be provided as a metadata to
          the new instance or a string containing a list of key-value pairs.
          Eg:  meta: "key1=value1,key2=value2"'
     required: false
     default: None
   wait:
     description:
        - If the module should wait for the instance to be created.
     required: false
     default: 'yes'
   timeout:
     description:
        - The amount of time the module should wait for the instance to get
          into active state.
     required: false
     default: 180
   config_drive:
     description:
        - Whether to boot the server with config drive enabled
     required: false
     default: 'no'
   userdata:
     description:
        - Opaque blob of data which is made available to the instance
     required: false
     default: None
   boot_from_volume:
     description:
        - Should the instance boot from a persistent volume created based on
          the image given. Mututally exclusive with boot_volume.
     required: false
     default: false
   volume_size:
     description:
        - The size of the volume to create in GB if booting from volume based
          on an image.
   boot_volume:
     description:
        - Volume name or id to use as the volume to boot from. Implies
          boot_from_volume. Mutually exclusive with image and boot_from_volume.
     required: false
     default: None
     aliases: ['root_volume']
   terminate_volume:
     description:
        - If true, delete volume when deleting instance (if booted from volume)
     default: false
   volumes:
     description:
       - A list of preexisting volumes names or ids to attach to the instance
     required: false
     default: []
   scheduler_hints:
     description:
        - Arbitrary key/value pairs to the scheduler for custom use
     required: false
     default: None
     version_added: "2.1"
   state:
     description:
       - Should the resource be present or absent.
     choices: [present, absent]
     default: present
   delete_fip:
     description:
       - When I(state) is absent and this option is true, any floating IP
         associated with the instance will be deleted along with the instance.
     required: false
     default: false
     version_added: "2.2"
   reuse_ips:
     description:
       - When I(auto_ip) is true and this option is true, the I(auto_ip) code
         will attempt to re-use unassigned floating ips in the project before
         creating a new one. It is important to note that it is impossible
         to safely do this concurrently, so if your use case involves
         concurrent server creation, it is highly recommended to set this to
         false and to delete the floating ip associated with a server when
         the server is deleted using I(delete_fip).
     required: false
     default: true
     version_added: "2.2"
requirements:
    - "python >= 2.6"
    - "shade"
'''

EXAMPLES = '''
- name: Create a new instance and attaches to a network and passes metadata to the instance
  os_server:
       state: present
       auth:
         auth_url: https://region-b.geo-1.identity.hpcloudsvc.com:35357/v2.0/
         username: admin
         password: admin
         project_name: admin
       name: vm1
       image: 4f905f38-e52a-43d2-b6ec-754a13ffb529
       key_name: ansible_key
       timeout: 200
       flavor: 4
       nics:
         - net-id: 34605f38-e52a-25d2-b6ec-754a13ffb723
         - net-name: another_network
       meta:
         hostname: test1
         group: uge_master

# Create a new instance in HP Cloud AE1 region availability zone az2 and
# automatically assigns a floating IP
- name: launch a compute instance
  hosts: localhost
  tasks:
    - name: launch an instance
      os_server:
        state: present
        auth:
          auth_url: https://region-b.geo-1.identity.hpcloudsvc.com:35357/v2.0/
          username: username
          password: Equality7-2521
          project_name: username-project1
        name: vm1
        region_name: region-b.geo-1
        availability_zone: az2
        image: 9302692b-b787-4b52-a3a6-daebb79cb498
        key_name: test
        timeout: 200
        flavor: 101
        security_groups: default
        auto_ip: yes

# Create a new instance in named cloud mordred availability zone az2
# and assigns a pre-known floating IP
- name: launch a compute instance
  hosts: localhost
  tasks:
    - name: launch an instance
      os_server:
        state: present
        cloud: mordred
        name: vm1
        availability_zone: az2
        image: 9302692b-b787-4b52-a3a6-daebb79cb498
        key_name: test
        timeout: 200
        flavor: 101
        floating_ips:
          - 12.34.56.79

# Create a new instance with 4G of RAM on Ubuntu Trusty, ignoring
# deprecated images
- name: launch a compute instance
  hosts: localhost
  tasks:
    - name: launch an instance
      os_server:
        name: vm1
        state: present
        cloud: mordred
        region_name: region-b.geo-1
        image: Ubuntu Server 14.04
        image_exclude: deprecated
        flavor_ram: 4096

# Create a new instance with 4G of RAM on Ubuntu Trusty on a Performance node
- name: launch a compute instance
  hosts: localhost
  tasks:
    - name: launch an instance
      os_server:
        name: vm1
        cloud: rax-dfw
        state: present
        image: Ubuntu 14.04 LTS (Trusty Tahr) (PVHVM)
        flavor_ram: 4096
        flavor_include: Performance

# Creates a new instance and attaches to multiple network
- name: launch a compute instance
  hosts: localhost
  tasks:
    - name: launch an instance with a string
      os_server:
        auth:
           auth_url: https://region-b.geo-1.identity.hpcloudsvc.com:35357/v2.0/
           username: admin
           password: admin
           project_name: admin
         name: vm1
         image: 4f905f38-e52a-43d2-b6ec-754a13ffb529
         key_name: ansible_key
         timeout: 200
         flavor: 4
         nics: "net-id=4cb08b20-62fe-11e5-9d70-feff819cdc9f,net-id=542f0430-62fe-11e5-9d70-feff819cdc9f..."

- name: Creates a new instance and attaches to a network and passes metadata to the instance
  os_server:
       state: present
       auth:
         auth_url: https://region-b.geo-1.identity.hpcloudsvc.com:35357/v2.0/
         username: admin
         password: admin
         project_name: admin
       name: vm1
       image: 4f905f38-e52a-43d2-b6ec-754a13ffb529
       key_name: ansible_key
       timeout: 200
       flavor: 4
       nics:
         - net-id: 34605f38-e52a-25d2-b6ec-754a13ffb723
         - net-name: another_network
       meta: "hostname=test1,group=uge_master"

- name:  Creates a new instance and attaches to a specific network
  os_server:
    state: present
    auth:
      auth_url: https://region-b.geo-1.identity.hpcloudsvc.com:35357/v2.0/
      username: admin
      password: admin
      project_name: admin
    name: vm1
    image: 4f905f38-e52a-43d2-b6ec-754a13ffb529
    key_name: ansible_key
    timeout: 200
    flavor: 4
      network: another_network

# Create a new instance with 4G of RAM on a 75G Ubuntu Trusty volume
- name: launch a compute instance
  hosts: localhost
  tasks:
    - name: launch an instance
      os_server:
        name: vm1
        state: present
        cloud: mordred
        region_name: ams01
        image: Ubuntu Server 14.04
        flavor_ram: 4096
        boot_from_volume: True
        volume_size: 75

# Creates a new instance with 2 volumes attached
- name: launch a compute instance
  hosts: localhost
  tasks:
    - name: launch an instance
      os_server:
        name: vm1
        state: present
        cloud: mordred
        region_name: ams01
        image: Ubuntu Server 14.04
        flavor_ram: 4096
        volumes:
        - photos
        - music
'''


def _exit_hostvars(module, cloud, server, changed=True):
    hostvars = meta.get_hostvars_from_server(cloud, server)
    module.exit_json(
        changed=changed, server=server, id=server.id, openstack=hostvars)


def _exit_hostvars_count(module, cloud, servers, changed=True):
    hostvars = []
    for server in servers:
        hostvars.append(meta.get_hostvars_from_server(cloud, server))
    server_ids = [server.id for server in servers]
    module.exit_json(
        changed=changed, servers=servers, ids=server_ids, openstack=hostvars)


def _parse_nics(nics):
    for net in nics:
        if isinstance(net, str):
            for nic in net.split(','):
                yield dict((nic.split('='),))
        else:
            yield net


def _network_args(module, cloud):
    args = []
    nics = module.params['nics']

    if not isinstance(nics, list):
        module.fail_json(msg='The \'nics\' parameter must be a list.')

    for net in _parse_nics(nics):
        if not isinstance(net, dict):
            module.fail_json(
                msg='Each entry in the \'nics\' parameter must be a dict.')

        if net.get('net-id'):
            args.append(net)
        elif net.get('net-name'):
            by_name = cloud.get_network(net['net-name'])
            if not by_name:
                module.fail_json(
                    msg='Could not find network by net-name: %s' %
                    net['net-name'])
            args.append({'net-id': by_name['id']})
        elif net.get('port-id'):
            args.append(net)
        elif net.get('port-name'):
            by_name = cloud.get_port(net['port-name'])
            if not by_name:
                module.fail_json(
                    msg='Could not find port by port-name: %s' %
                    net['port-name'])
            args.append({'port-id': by_name['id']})
    return args


def _delete_server(module, cloud):
    try:
        cloud.delete_server(
            module.params['name'], wait=module.params['wait'],
            timeout=module.params['timeout'],
            delete_ips=module.params['delete_fip'])
    except Exception as e:
        module.fail_json(msg="Error in deleting vm: %s" % e.message)
    module.exit_json(changed=True, result='deleted')


def _delete_server_count(module, cloud):
    changed = False
    server_name = module.params['name']
    count = module.params['count']
    for i in range(1, count+1):
        server = server_name+str(i)
        server = cloud.get_server(server)
        if server:
            try:
                cloud.delete_server(
                    server, wait=module.params['wait'],
                    timeout=module.params['timeout'],
                    delete_ips=module.params['delete_fip'])
            except Exception as e:
                module.fail_json(msg="Error in deleting vm: %s" % e.message)
            changed = True
    module.exit_json(changed=changed, result='deleted')


def _create_server(module, cloud):
    flavor = module.params['flavor']
    flavor_ram = module.params['flavor_ram']
    flavor_include = module.params['flavor_include']

    image_id = None
    if not module.params['boot_volume']:
        image_id = cloud.get_image_id(
            module.params['image'], module.params['image_exclude'])
        if not image_id:
            module.fail_json(msg="Could not find image %s" %
                             module.params['image'])

    if flavor:
        flavor_dict = cloud.get_flavor(flavor)
        if not flavor_dict:
            module.fail_json(msg="Could not find flavor %s" % flavor)
    else:
        flavor_dict = cloud.get_flavor_by_ram(flavor_ram, flavor_include)
        if not flavor_dict:
            module.fail_json(msg="Could not find any matching flavor")

    nics = _network_args(module, cloud)

    if isinstance(module.params['meta'], str):
        metas = {}
        for kv_str in module.params['meta'].split(","):
            k, v = kv_str.split("=")
            metas[k] = v
        module.params['meta'] = metas

    bootkwargs = dict(
        name=module.params['name'],
        image=image_id,
        flavor=flavor_dict['id'],
        nics=nics,
        meta=module.params['meta'],
        security_groups=module.params['security_groups'],
        userdata=module.params['userdata'],
        config_drive=module.params['config_drive'],
    )
    for optional_param in (
            'key_name', 'availability_zone', 'network',
            'scheduler_hints', 'volume_size', 'volumes'):
        if module.params[optional_param]:
            bootkwargs[optional_param] = module.params[optional_param]

    server = cloud.create_server(
        ip_pool=module.params['floating_ip_pools'],
        ips=module.params['floating_ips'],
        auto_ip=module.params['auto_ip'],
        boot_volume=module.params['boot_volume'],
        boot_from_volume=module.params['boot_from_volume'],
        terminate_volume=module.params['terminate_volume'],
        reuse_ips=module.params['reuse_ips'],
        wait=module.params['wait'], timeout=module.params['timeout'],
        **bootkwargs
    )

    _exit_hostvars(module, cloud, server)


def _create_server_count(module, cloud):
    count = module.params['count']
    name = module.params['name']
    flavor = module.params['flavor']
    flavor_ram = module.params['flavor_ram']
    flavor_include = module.params['flavor_include']
    image_id = None
    changed = False

    if not module.params['boot_volume']:
        image_id = cloud.get_image_id(
            module.params['image'], module.params['image_exclude'])

    if flavor:
        flavor_dict = cloud.get_flavor(flavor)
        if not flavor_dict:
            module.fail_json(msg="Could not find flavor %s" % flavor)
    else:
        flavor_dict = cloud.get_flavor_by_ram(flavor_ram, flavor_include)
        if not flavor_dict:
            module.fail_json(msg="Could not find any matching flavor")

    nics = _network_args(module, cloud)

    if isinstance(module.params['meta'], str):
        metas = {}
        for kv_str in module.params['meta'].split(","):
            k, v = kv_str.split("=")
            metas[k] = v
        module.params['meta'] = metas

    server_names = [ name+str(i) for i in range(1, count+1) ]
    servers = []

    for server_name in server_names:
        server = cloud.get_server(server_name)
        if server:
            if server.status not in ('ACTIVE', 'SHUTOFF', 'PAUSED', 'SUSPENDED'):
                module.fail_json(
                    msg="One or more instances in specified count us available \
                         but not Active state: " + server.status)
            servers.append(server)
        else:
            bootkwargs = dict(
                name=server_name,
                image=image_id,
                flavor=flavor_dict['id'],
                nics=nics,
                meta=module.params['meta'],
                security_groups=module.params['security_groups'],
                userdata=module.params['userdata'],
                config_drive=module.params['config_drive'],
            )
            for optional_param in (
                    'key_name', 'availability_zone', 'network',
                    'scheduler_hints', 'volume_size', 'volumes'):
                if module.params[optional_param]:
                    bootkwargs[optional_param] = module.params[optional_param]
            server = cloud.create_server(
                ip_pool=module.params['floating_ip_pools'],
                ips=module.params['floating_ips'],
                auto_ip=module.params['auto_ip'],
                boot_volume=module.params['boot_volume'],
                boot_from_volume=module.params['boot_from_volume'],
                terminate_volume=module.params['terminate_volume'],
                reuse_ips=module.params['reuse_ips'],
                wait=module.params['wait'], timeout=module.params['timeout'],
                **bootkwargs
            )
            servers.append(server)
            changed = True

    _exit_hostvars_count(module, cloud, servers, changed=changed)


def _delete_floating_ip_list(cloud, server, extra_ips):
    for ip in extra_ips:
        cloud.nova_client.servers.remove_floating_ip(
            server=server.id, address=ip)


def _check_floating_ips(module, cloud, server):
    changed = False

    auto_ip = module.params['auto_ip']
    floating_ips = module.params['floating_ips']
    floating_ip_pools = module.params['floating_ip_pools']

    if floating_ip_pools or floating_ips or auto_ip:
        ips = openstack_find_nova_addresses(server.addresses, 'floating')
        if not ips:
            # If we're configured to have a floating but we don't have one,
            # let's add one
            server = cloud.add_ips_to_server(
                server,
                auto_ip=auto_ip,
                ips=floating_ips,
                ip_pool=floating_ip_pools,
                wait=module.params['wait'],
                timeout=module.params['timeout'],
            )
            changed = True
        elif floating_ips:
            # we were configured to have specific ips, let's make sure we have
            # those
            missing_ips = []
            for ip in floating_ips:
                if ip not in ips:
                    missing_ips.append(ip)
            if missing_ips:
                server = cloud.add_ip_list(server, missing_ips,
                                           wait=module.params['wait'],
                                           timeout=module.params['timeout'])
                changed = True
            extra_ips = []
            for ip in ips:
                if ip not in floating_ips:
                    extra_ips.append(ip)
            if extra_ips:
                _delete_floating_ip_list(cloud, server, extra_ips)
                changed = True
    return (changed, server)


def _get_server_state(module, cloud):
    state = module.params['state']
    server = cloud.get_server(module.params['name'])
    if server and state == 'present':
        if server.status not in ('ACTIVE', 'SHUTOFF', 'PAUSED', 'SUSPENDED'):
            module.fail_json(
                msg="The instance is available but not Active state: "
                    + server.status)
        (ip_changed, server) = _check_floating_ips(module, cloud, server)
        _exit_hostvars(module, cloud, server, ip_changed)
    if server and state == 'absent':
        return True
    if state == 'absent':
        module.exit_json(changed=False, result="not present")
    return True


def main():

    argument_spec = openstack_full_argument_spec(
        name                            = dict(required=True),
        image                           = dict(default=None),
        image_exclude                   = dict(default='(deprecated)'),
        flavor                          = dict(default=None),
        flavor_ram                      = dict(default=None, type='int'),
        flavor_include                  = dict(default=None),
        key_name                        = dict(default=None),
        security_groups                 = dict(default=['default'], type='list'),
        network                         = dict(default=None),
        nics                            = dict(default=[], type='list'),
        meta                            = dict(default=None, type='raw'),
        userdata                        = dict(default=None, aliases=['user_data']),
        config_drive                    = dict(default=False, type='bool'),
        auto_ip                         = dict(default=True, type='bool', aliases=['auto_floating_ip', 'public_ip']),
        floating_ips                    = dict(default=None, type='list'),
        floating_ip_pools               = dict(default=None, type='list'),
        volume_size                     = dict(default=False, type='int'),
        boot_from_volume                = dict(default=False, type='bool'),
        boot_volume                     = dict(default=None, aliases=['root_volume']),
        terminate_volume                = dict(default=False, type='bool'),
        volumes                         = dict(default=[], type='list'),
        scheduler_hints                 = dict(default=None, type='dict'),
        state                           = dict(default='present', choices=['absent', 'present']),
        delete_fip                      = dict(default=False, type='bool'),
        reuse_ips                       = dict(default=True, type='bool'),
        count                           = dict(default=None, type='int'),
    )
    module_kwargs = openstack_module_kwargs(
        mutually_exclusive=[
            ['auto_ip', 'floating_ips'],
            ['auto_ip', 'floating_ip_pools'],
            ['floating_ips', 'floating_ip_pools'],
            ['flavor', 'flavor_ram'],
            ['image', 'boot_volume'],
            ['boot_from_volume', 'boot_volume'],
            ['nics', 'network'],
        ],
        required_if=[
            ('boot_from_volume', True, ['volume_size', 'image']),
        ],
    )
    module = AnsibleModule(argument_spec, **module_kwargs)

    if not HAS_SHADE:
        module.fail_json(msg='shade is required for this module')

    state = module.params['state']
    image = module.params['image']
    boot_volume = module.params['boot_volume']
    flavor = module.params['flavor']
    flavor_ram = module.params['flavor_ram']
    count = module.params['count']

    if state == 'present':
        if not (image or boot_volume):
            module.fail_json(
                msg="Parameter 'image' or 'boot_volume' is required "
                    "if state == 'present'"
            )
        if not flavor and not flavor_ram:
            module.fail_json(
                msg="Parameter 'flavor' or 'flavor_ram' is required "
                    "if state == 'present'"
            )

    try:
        cloud_params = dict(module.params)
        cloud_params.pop('userdata', None)
        cloud = shade.openstack_cloud(**cloud_params)

        if state == 'present' and not count:
            _get_server_state(module, cloud)
            _create_server(module, cloud)
        elif state == 'present' and count:
            _create_server_count(module, cloud)
        elif state == 'absent' and not count:
            _get_server_state(module, cloud)
            _delete_server(module, cloud)
        elif state == 'absent' and count:
            _delete_server_count(module, cloud)
    except shade.OpenStackCloudException as e:
        module.fail_json(msg=str(e), extra_data=e.extra_data)


# this is magic, see lib/ansible/module_common.py
from ansible.module_utils.basic import *
from ansible.module_utils.openstack import *
if __name__ == '__main__':
    main()



#!/usr/bin/env python

from linchpin.api import LinchpinAPI


class LinchpinCli(LinchpinAPI):

    def __init__(self, ctx):
        """
        Set some variables, pass to parent class
        """

        LinchpinAPI.__init__(self, ctx)






import yaml
import json
import jsonschema
import io
from jsonschema import validate
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


def get_sections(ini):
    ini = ini.decode('utf-8') 
    buf = io.StringIO(ini)
    config = ConfigParser(allow_no_value=True)
    config.readfp(buf)
    sections = config.sections()
    return sections



#!/usr/bin/env python
import abc
import StringIO
from ansible import errors

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


class InventoryFilter(object):
    __metaclass__ = abc.ABCMeta

    def __init__(self):
        self.config = ConfigParser(allow_no_value=True)

    @abc.abstractmethod
    def get_host_ips(self, topo):
        pass

    @abc.abstractmethod
    def get_inventory(self, topo, layout):
        pass

    def get_layout_hosts(self, inv):
        count = 0
        for host_group in inv['hosts']:
            if 'count' in inv['hosts'][host_group]:
                count += inv['hosts'][host_group]['count']
            else:
                count += 1
        return count

    def get_layout_host_groups(self, inv):
        """
        get all the list of host groups in layout
        """
        host_groups = []
        for host in inv['hosts']:
            if "host_groups" in inv["hosts"][host].keys():
                host_groups.extend(inv['hosts'][host]["host_groups"])
        return list(set(host_groups))

    def add_sections(self, section_list):
        for section in section_list:
            self.config.add_section(section)
        # adding a default section all
        if "all" not in self.config.sections():
            self.config.add_section("all")

    def set_children(self, inv):
        if 'host_groups' not in inv.keys():
            return
        for host_group in inv['host_groups']:
            if "children" in inv['host_groups'][host_group]:
                self.config.add_section(host_group+":"+"children")
                for child in inv['host_groups'][host_group]['children']:
                    self.config.set(host_group+":"+"children", child)

    def set_vars(self, inv):
        if 'host_groups' not in inv.keys():
            return
        for host_group in inv['host_groups']:
            if "vars" in inv['host_groups'][host_group]:
                self.config.add_section(host_group+":"+"vars")
                for var in inv['host_groups'][host_group]['vars']:
                    grp_vars = inv['host_groups'][host_group]['vars'][var]
                    grp_vars = str(grp_vars)
                    self.config.set(host_group + ":" + "vars", var, grp_vars)

    def add_ips_to_groups(self, inven_hosts, layout):
        # create a ip to host mapping based on count
        ip_to_host = {}
        host_grps = self.get_layout_host_groups(layout)
        inven_hosts.reverse()
        for host_name in layout['hosts']:
            if 'count' in layout['hosts'][host_name]:
                count = layout['hosts'][host_name]['count']
            else:
                count = 1
            host_list = []
            for i in range(0, count):
                item = inven_hosts.pop()
                host_list.append(item)
            ip_to_host[host_name] = host_list
        # add ips to the respective host groups in inventory
        for host_name in layout['hosts']:
            host_ips = ip_to_host[host_name]
            for ip in host_ips:
                for host_group in layout['hosts'][host_name]['host_groups']:
                    self.config.set(host_group, ip)
                    self.config.set("all", ip)

    def add_common_vars(self, host_groups, layout):
        # defaults common_vars to [] when they doesnot exist
        host_groups.append("all")
        common_vars = layout['vars'] if 'vars' in layout.keys() else []
        for group in host_groups:
            items = dict(self.config.items(group)).keys()
            self.config.remove_section(group)
            self.config.add_section(group)
            for item in items:
                host_string = item
                for var in common_vars:
                    if common_vars[var] == "__IP__":
                        host_string += " " + var + "=" + item + " "
                self.config.set(group, host_string)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Author: Clint Savage- @herlo <herlo@redhat.com>
#
# Provision a dummy server. Useful for testing the linchpin api provisioner
# without using actual resources.
#

#---- Documentation Start ----------------------------------------------------#
DOCUMENTATION = '''
---
version_added: "0.1"
module: dummy
short_description: Dummy instance managemer
description:
  - This module allows a user to manage any number of dummy systems.
options:
  name:
    description:
      Given name for task
  state:
    description:
      Allocate or Deallocate instances
    required: true

requirements: []
author: Clint Savage - @herlo
'''

EXAMPLES = '''
- name: "provision a dummy node"
  dummy:
    state: present
    name: my_dummy_node
  register: herlo-dummy-node

# teardown node named my_dummy_node
- name: "teardown dummy node"
  duffy:
    state: absent
    name: my_dummy_node

'''

#---- Logic Start ------------------------------------------------------------#
import os, re, sys, json, random, string, tempfile

from ansible.constants import mk_boolean
from ansible.module_utils.basic import *


class Dummy:

    def __init__(self):

        self.DUMMY_FILE = '{0}/dummy.hosts'.format(tempfile.gettempdir())
        pass

#    def generate_hostname(self, minlen=5, maxlen=20):
#        """
#        Generate a random hostname between minlen and maxlen, appending
#        '.example.net' (eg. oskpijsrss7.example.net)
#
#        :param minlen: minimum length of the hostname (default: 5)
#
#        :param maxlen: maximum length of the hostname (default: 20)
#        """
#
#        len = random.randint(minlen, maxlen)
#
#        host_str = ''.join(
#            random.choice(string.ascii_lowercase) for _ in range(len -1))
#
#        host_str = '{0}{1}.example.net'.format(
#                host_str, random.choice(string.digits))
#
#        return host_str


    def allocate(self, name, count=1):
        """
        Create some dummy host as if these systems existed.
        Write the host out to the dummy.hosts temporary file to avoid
        duplication.
        """

        changed = False

        hosts = []

        for i in range(count):
            host = '{0}-{1}.example.net'.format(name, i)
            with open(self.DUMMY_FILE, 'a+') as f:
                if not any('{0}\n'.format(host) in line for line in f):
                    f.write('{0}\n'.format(host))
                    changed = True
            hosts.append(host)

        return (changed, hosts)


    def deallocate(self, name, count=1):
        """
        Deallocate some dummy host.
        Remove the host from the dummy.hosts temporary file.
        """

        changed = False
        out = []
        hosts_to_del = []

        if not os.path.exists(self.DUMMY_FILE):
            return changed

        with open(self.DUMMY_FILE, 'r+') as f:
            lines = f.readlines()

            hostre = '^{0}-\d.example.net'.format(name)
            for line in lines:
                if re.search(hostre, line):
                    hosts_to_del.append(line)
                    changed = True

        num2del =  len(lines)-count
        out = hosts_to_del[0:num2del]

        with open(self.DUMMY_FILE, 'w') as f:
            f.writelines(out)

        return changed


    def execute(self, module):

        json_output = {}
        name = module.params['name']
        state = module.params['state']
        count = module.params['count']

        # allocate some systems if state is 'present' :)
        if state == 'present':

            changed, hosts = self.allocate(name, count)
            json_output['changed'] = changed
            json_output['hosts'] = hosts
            json_output['dummy_file'] = self.DUMMY_FILE

            return json_output

        elif state == 'absent':

            status = self.deallocate(name, count)
            json_output['changed'] = status
            json_output['dummy_file'] = self.DUMMY_FILE


        return json_output

def main():

    module = AnsibleModule(
        argument_spec=dict(
            name = dict(type='str'),
            count = dict(default=1, type='int'),
            state = dict(choices=['present', 'absent']),
        ),
    )

    try:
        d = Dummy()
        execute_output = d.execute(module)

        json_output = {}
        host = execute_output.get('host')
        changed = execute_output.get('changed')
        if host or changed is not None:
            json_output['changed'] = True
            json_output.update(execute_output)
        else:
            json_output['changed'] = False

        module.exit_json(**json_output)
    except Exception as e:
        module.fail_json(msg=str(e))


#---- Import Ansible Utilities (Ansible Framework) ---------------------------#
main()



import os
import yaml
import json
import ansible
from ansible import utils
from collections import namedtuple
from ansible import utils
from ansible.parsing.dataloader import DataLoader
from ansible.vars import VariableManager
from ansible.inventory import Inventory
from ansible.executor.playbook_executor import PlaybookExecutor
from ansible.plugins.callback import CallbackBase
from action_manager import ActionManager
from cerberus import Validator

from linchpin.exceptions import HookError


class AnsibleActionManager(ActionManager):

    def __init__(self, name, action_data, target_data, **kwargs):
        
        """
        AnsibleActionManager constructor 
        :param name: Name of Action Manager , ( ie., ansible)
        :param action_data: dictionary of action_block consists of set of actions
        example:
        - name: nameofthehook
          type: ansible
          actions:
            - playbook: test_playbook.yaml
              vars: test_var.yaml
              extra_vars: { "test_var": "postdowntask"}
        :param target_data: Target specific data defined in PinFile
        :param kwargs: anyother keyword args passed as metadata
        """

        self.name = name
        self.action_data = action_data
        self.target_data = target_data
        self.context = kwargs.get("context", True)
        self.kwargs = kwargs

    def validate(self):

        """
        Validates the action_block based on the cerberus schema
        example:: ansible_action_block:::: 
        - name: build_openshift_cluster
          type: ansible
          actions:
            - playbook: test_playbook.yaml
              vars: test_var.yaml
              extra_vars: { "testvar": "world"}
        """

        schema= {
        'name': {'type': 'string', 'required': True },
        'type': { 'type': 'string', 'allowed': ['ansible']},
        'path': {'type': 'string', 'required': False},
        'context': {'type': 'boolean', 'required': False},
        'actions': { 'type': 'list',
                     'schema': {
                         'type': 'dict',
                         'schema': {
                             'playbook': {'type': 'string', 'required': True},
                             'vars': {'type': 'string', 'required': False},
                             'extra_vars': {'type': 'dict', 'required': False}
                         }
                     },
                     'required': True
                   }
        }
        v = Validator(schema)
        status = v.validate(self.action_data)
        if not status:
            raise HookError("Invalid syntax: LinchpinHook:"+str((v.errors)))
        else:
            return status

    def load(self):
        
        """
        Loads the ansible specific managers and loaders
        """

        self.loader = DataLoader()
        self.variable_manager = VariableManager()
        self.passwords = {}
        if self.target_data.has_key("inventory_file") and self.context:
            self.inventory = Inventory(loader=self.loader,
                                       variable_manager=self.variable_manager,
                                       host_list=self.target_data["inventory_file"])
        else:
            self.inventory = Inventory(loader=self.loader,
                                       variable_manager=self.variable_manager,
                                       host_list=["localhost"])
        Options = namedtuple('Options', ['listtags',
                                         'listtasks',
                                         'listhosts',
                                         'syntax',
                                         'connection',
                                         'module_path',
                                         'forks',
                                         'remote_user',
                                         'private_key_file',
                                         'ssh_common_args',
                                         'ssh_extra_args',
                                         'sftp_extra_args',
                                         'scp_extra_args',
                                         'become',
                                         'become_method',
                                         'become_user',
                                         'verbosity',
                                         'check'])
        utils.VERBOSITY = 4
        self.options = Options(listtags=False,
                          listtasks=False,
                          listhosts=False,
                          syntax=False,
                          connection='ssh',
                          module_path="",
                          forks=100,
                          remote_user='root',
                          private_key_file=None,
                          ssh_common_args=None,
                          ssh_extra_args=None,
                          sftp_extra_args=None,
                          scp_extra_args=None,
                          become=False,
                          become_method="sudo",
                          become_user='root',
                          verbosity=utils.VERBOSITY,
                          check=False)


    def get_ansible_runner(self, playbook_path, extra_vars):
        
        """
        Fetches ansible runner based on playbook_path and extra_vars
        :param playbook_path: path to playbook
        :param extra_vars: variables to be passed
        """

        self.variable_manager.extra_vars = extra_vars
        # though verbosity through api doesnot work
        pbex = PlaybookExecutor(playbooks=[playbook_path],
                                inventory=self.inventory,
                                variable_manager=self.variable_manager,
                                loader=self.loader,
                                options=self.options,
                                passwords=self.passwords)
        return pbex

    def get_ctx_params(self):
        
        """
        Reformats the ansible specific variables
        """

        ctx_params = {}
        ctx_params["resource_file"] = self.target_data.get("resource_file",None)
        ctx_params["layout_file"] = self.target_data.get("layout_file",None)
        ctx_params["inventory_file"] = self.target_data.get("inventory_file",None)
        return ctx_params


    def execute(self):
        
        """
        Executes the action_block in the PinFile
        """

        self.load()
        extra_vars = {}
        for action in self.action_data["actions"]:
            path = self.action_data["path"]
            playbook = action.get("playbook")
            if not(os.path.isfile(playbook)):
                playbook = "{0}/{1}".format(
                            path,
                            playbook
                            )
            if action.has_key("vars"):
                var_file = "{0}/{1}".format(
                           path,
                           action.get("vars")
                           )
                ext = var_file.split(".")[-1]
                extra_vars = open(var_file,"r").read()
                if ("yaml" in ext) or ("yml" in ext):
                    extra_vars = yaml.load(extra_vars)
                else:
                   extra_vars = json.loads(extra_vars)
            e_vars = action.get("extra_vars", {})
            extra_vars.update(e_vars)
            if self.context:
                #extra_vars["linchpin_context"] = self.get_ctx_params()
                extra_vars.update(self.get_ctx_params())

            pbex = self.get_ansible_runner(playbook, extra_vars)
            results = pbex.run()
